@@ -1,7 +1,7 @@
 /*
- *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2007-2008, RW Penney
+ *  Methods for encryption/security mechanisms for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2007-2009, RW Penney
  */
 
 /*
@@ -39,32 +39,50 @@
 #endif
 
 
+typedef struct {
+    unsigned fversion;
+} blti_overrides_t;
+
 
 /*
- *  ==== built-in sha1/blowfish key-management routines ====
+ *  ==== Built-in sha1/blowfish key-management routines ====
  */
 
 
 /*
- *  keyfile format is:
+ *  Keyfile format is:
  *      char magic[7]="cm-blti";
- *      char version;
+ *      uchar version;
  *      uint16{LSB-first} keylength;
+ *      uint32{LSB-first} hash-iterations (version>=1);
  *      char salt[kmblti_saltlen];
  *      [64-bit block][64-bit block][...];
  *      uint64{LSB-first} xor-checksum of key
  */
 
 static const char kmblti_magstr[]="cm-blti";
-static const char kmblti_version = (char)0;
+static const unsigned char kmblti_version = (char)1;
 static const size_t kmblti_maglen = 7;  /* = strlen(kmblti_magstr) */
 enum {
-    kmblti_saltlen = 10
+    kmblti_saltlen = 10,
+    kmblti_default_iterations = 1 << 14
 };
 
 
-static cm_bf_ctxt_t *kmblti_initcipher(const unsigned char *salt,
+static int kmblti_checkversion(unsigned char fversion)
+{
+    if (fversion > kmblti_version) {
+        fprintf(stderr, "Bad keyfile version [%d]\n", (int)fversion);
+        return ERR_BADFILE;
+    }
+
+    return ERR_NOERROR;
+}
+
+
+static cm_bf_ctxt_t *kmblti_initcipher_v0(const unsigned char *salt,
         const char *pass, size_t passlen, uint32_t iv[2])
+    /* Initialize cipher key (for file-format version-0) */
 {   cm_bf_ctxt_t *ctxt;
     cm_sha1_ctxt_t *md;
     unsigned char *ckey=NULL;
@@ -83,7 +101,7 @@ static cm_bf_ctxt_t *kmblti_initcipher(const unsigned char *salt,
     cm_sha1_final(md, &ckey, &ckeysz);
     cm_sha1_free(md);
 
-    /* Initialized Blowfish cipher with hashed password: */
+    /* Initialize Blowfish cipher with hashed password: */
     ctxt = cm_bf_init(ckey, ckeysz);
     sec_free((void*)ckey);
 
@@ -91,6 +109,27 @@ static cm_bf_ctxt_t *kmblti_initcipher(const unsigned char *salt,
 }
 
 
+static cm_bf_ctxt_t *kmblti_initcipher_v1(const unsigned char *salt,
+        const char *pass, size_t passlen, uint32_t iterations, uint32_t iv[2])
+    /* Initialize cipher key (for file-format version-1) */
+{   cm_bf_ctxt_t *ctxt;
+    unsigned char *ckey=NULL;
+    const size_t ckeysz = 56;
+
+    cm_pwd_fortify(pass, iterations, salt, (size_t)kmblti_saltlen,
+                    &ckey, ckeysz);
+
+    iv[0] = pack_uint32(ckey + 48);
+    iv[1] = pack_uint32(ckey + 52);
+
+    /* Initialize Blowfish cipher with hashed password: */
+    ctxt = cm_bf_init(ckey, ckeysz - 8);
+    sec_free((void*)ckey);
+
+    return ctxt;
+}
+
+
 static int kmblti_init_algs(void)
 {
     /* Nothing needed */
@@ -105,95 +144,112 @@ static int kmblti_free_algs(void)
 }
 
 
-static void kmblti_mk_default(keyinfo_t *keyinfo)
-{
-    if (keyinfo == NULL) return;
-
-    if (keyinfo->digestalg == NULL) {
-        keyinfo->digestalg = cm_strdup("sha1");
-    }
-
-    if (keyinfo->cipheralg == NULL) {
-        keyinfo->cipheralg = cm_strdup("blowfish-cbc");
-    }
-}
-
-
-static int kmblti_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
-{   char buff[32];
+static int kmblti_bind(bound_tgtdefn_t *bound, FILE *fp_key)
+{   keyinfo_t *keyinfo = &bound->tgt->key;
+    const char *fmtptr;
+    char buff[32];
+    int compat = 1;     /* Be prepared to act as default key-manager */
 
     if (keyinfo->format != NULL) {
-        return (strcmp(keyinfo->format, "builtin") == 0);
+        fmtptr = keyinfo->format;
+        compat = cm_startswith(&fmtptr, "builtin");
+        if (*fmtptr == ':') {
+            /* Extract file-format version from suffix: */
+            blti_overrides_t *bltior;
+            bltior = (blti_overrides_t*)malloc(sizeof(blti_overrides_t));
+            bltior->fversion = atoi(fmtptr + 1);
+            bound->km_data = (void*)bltior;
+        } else if (*fmtptr != '\0') compat = 0;
     } else {
         if (fp_key != NULL) {
             /* Check header of existing key-file: */
             buff[0] = '\0';
-            (void)fread((void*)buff, kmblti_maglen, (size_t)1, fp_key);
-            return (strncmp(buff, kmblti_magstr, kmblti_maglen) == 0);
+            compat = (cm_fread((void*)buff, kmblti_maglen, fp_key) == 0
+                        && strncmp(buff, kmblti_magstr, kmblti_maglen) == 0);
         }
     }
 
-    /* Be prepared to act as default key-manager: */
-    return 1;
+    if (compat) {
+        if (keyinfo->digestalg == NULL) {
+            keyinfo->digestalg = cm_strdup("sha1");
+        }
+
+        if (keyinfo->cipheralg == NULL) {
+            keyinfo->cipheralg = cm_strdup("blowfish-cbc");
+        }
+    }
+
+    return compat;
 }
 
 
-static unsigned kmblti_get_properties(const keyinfo_t *keyinf)
+static unsigned kmblti_get_properties(const bound_tgtdefn_t *boundtgt)
 {
-    return KM_PROP_HASPASSWD;
+    return (KM_PROP_HASPASSWD | KM_PROP_NEEDSKEYFILE);
 }
 
 
-static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmblti_get_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key,
-	    km_overrides_t *overrides)
+            unsigned char **key, int *keylen, FILE *fp_key)
     /* Extract key from sha1/blowfish encrypted file */
-{   cm_bf_ctxt_t *ctxt;
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    cm_bf_ctxt_t *ctxt;
     enum { BUFFSZ = 512 };
-    unsigned char hbuff[kmblti_maglen+4], salt[kmblti_saltlen],
-                *buff=NULL, *bptr;
-    uint32_t iv[2], cv[2], pl[2];
+    unsigned char *hbuff=NULL, fversion, salt[kmblti_saltlen],
+            *buff=NULL, *bptr;
+    uint32_t iv[2], cv[2], pl[2], iterations = kmblti_default_iterations;
     char *passwd=NULL;
     uint32_t chksum, chksum0;
-    int cnt, eflag=ERR_NOERROR;
+    int cnt, rd_errs=0, eflag=ERR_NOERROR;
 
     *key = NULL; *keylen = 0;
 
-    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 0, 0);
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 0, 0);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     /* Read key header: */
+    hbuff = (unsigned char*)malloc((size_t)(kmblti_maglen + 4));
     hbuff[0] = '\0';
-    (void)fread((void*)hbuff, (size_t)1, kmblti_maglen, fp_key);
+    rd_errs += cm_fread((void*)hbuff, kmblti_maglen, fp_key);
     if (strncmp((const char*)hbuff, kmblti_magstr, kmblti_maglen) != 0) {
         fprintf(stderr, "Bad keyfile format (builtin)\n");
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    fread((void*)hbuff, (size_t)1, (size_t)1, fp_key);
-    if (hbuff[0] != '\0') {
-        fprintf(stderr, "Bad keyfile version [%d]\n", (int)hbuff[0]);
-        eflag = ERR_BADFILE;
-        goto bail_out;
+    rd_errs += cm_fread((void*)&fversion, (size_t)1, fp_key);
+    eflag = kmblti_checkversion(fversion);
+    if (eflag != ERR_NOERROR) goto bail_out;
+    rd_errs += cm_fread((void*)hbuff, (size_t)2, fp_key);
+    *keylen = pack_uint16(hbuff);
+
+    /* Read iteration-count from keyfile: */
+    if (fversion == 1) {
+        rd_errs += cm_fread((void*)hbuff, (size_t)4, fp_key);
+        iterations = pack_uint32(hbuff);
     }
-    fread((void*)hbuff, (size_t)2, (size_t)1, fp_key);
-    *keylen = ((unsigned)hbuff[0]) | (((unsigned)hbuff[1]) << 8);
 
     /* Read salt from keyfile: */
-    fread((void*)salt, (size_t)1, sizeof(salt), fp_key);
+    rd_errs += cm_fread((void*)salt, sizeof(salt), fp_key);
 
     /* Read encrypted key from keyfile: */
-    ctxt = kmblti_initcipher(salt, passwd, strlen(passwd), iv);
+    switch (fversion) {
+        case 0:
+            ctxt = kmblti_initcipher_v0(salt, passwd, strlen(passwd), iv);
+            break;
+        case 1: /* Fall-through: */
+        default:
+            ctxt = kmblti_initcipher_v1(salt, passwd, strlen(passwd),
+                                        iterations, iv);
+            break;
+    }
     cnt = km_aug_keysz((unsigned)*keylen, 8u) / 8;
     buff = sec_realloc(buff, (size_t)(cnt * 8));
-    fread((void*)buff, (size_t)8, (size_t)cnt, fp_key);
+    rd_errs += cm_fread((void*)buff, (size_t)(8 * cnt), fp_key);
     bptr = buff;
     while (cnt--) {
-        cv[0] = (((uint32_t)bptr[7]) << 24) | (((uint32_t)bptr[6]) << 16)
-                | (((uint32_t)bptr[5]) << 8) | ((uint32_t)bptr[4]);
-        cv[1] = (((uint32_t)bptr[3]) << 24) | (((uint32_t)bptr[2]) << 16)
-                | (((uint32_t)bptr[1]) << 8) | ((uint32_t)bptr[0]);
+        cv[0] = pack_uint32(bptr+4);
+        cv[1] = pack_uint32(bptr);
 
         /* Apply cipher block-chaining: */
         pl[0] = cv[0];
@@ -238,7 +294,7 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
     *key = (unsigned char*)sec_realloc((void*)*key, (size_t)*keylen);
     memcpy(*key, buff, (size_t)*keylen);
 
-    if (ferror(fp_key) != 0) {
+    if (rd_errs > 0 || ferror(fp_key) != 0) {
         fprintf(stderr, _("Key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
         eflag = ERR_BADFILE;
@@ -248,42 +304,65 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
 
     if (buff != NULL) sec_free((void*)buff);
     if (passwd != NULL) sec_free((void*)passwd);
+    if (hbuff != NULL) free((void*)hbuff);
 
     return eflag;
 }
 
 
-static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmblti_put_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key,
-	    const km_overrides_t *overrides)
+            const unsigned char *key, const int keylen, FILE *fp_key)
     /* Store key in sha1/blowfish encrypted file */
 {   cm_bf_ctxt_t *ctxt;
-    unsigned char salt[kmblti_saltlen], hbuff[2], *buff=NULL, *bptr;
-    uint32_t iv[2], cv[2];
+    unsigned char fversion, salt[kmblti_saltlen], hbuff[4],
+                *buff=NULL, *bptr;
+    uint32_t iv[2], cv[2], iterations = kmblti_default_iterations;
     char *passwd=NULL;
-    size_t buffsz, wrcnt;
-    int cnt, eflag=ERR_NOERROR;
+    size_t buffsz;
+    blti_overrides_t *bltior=NULL;
+    int cnt, wr_errs=0, eflag=ERR_NOERROR;
 
-    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 1, 1);
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 1, 1);
+    if (eflag != ERR_NOERROR) goto bail_out;
+
+    fversion = kmblti_version;
+    if (boundtgt->km_data != NULL) {
+        bltior = (blti_overrides_t*)boundtgt->km_data;
+        fversion = bltior->fversion;
+    }
+    eflag = kmblti_checkversion(fversion);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     /* Write key header: */
-    wrcnt = 0;
-    wrcnt += fwrite((const void*)kmblti_magstr, kmblti_maglen, (size_t)1, fp_key);
-    wrcnt += fwrite((const void*)&kmblti_version, (size_t)1, (size_t)1, fp_key);
-    hbuff[0] = (keylen & 0xff); hbuff[1] = (keylen & 0xff00) >> 8;
-    wrcnt += fwrite((const void*)hbuff, (size_t)2, (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)kmblti_magstr, kmblti_maglen, fp_key);
+    wr_errs += cm_fwrite((const void*)&fversion, (size_t)1, fp_key);
+    unpack_uint16(hbuff, (uint16_t)keylen);
+    wr_errs += cm_fwrite((const void*)hbuff, (size_t)2, fp_key);
+
+    /* Write iteration-count: */
+    if (fversion == 1) {
+        unpack_uint32(hbuff, iterations);
+        wr_errs += cm_fwrite((const void*)hbuff, (size_t)4, fp_key);
+    }
 
     /* Generate salt & record in key-file: */
     get_randkey(salt, sizeof(salt));
-    wrcnt += fwrite((const void*)salt, sizeof(salt), (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)salt, sizeof(salt), fp_key);
 
     /* Augment key with simple checksum: */
     buff = km_aug_key(key, (unsigned)keylen, 8u, &buffsz);
 
     /* Write encrypted key into keyfile: */
-    ctxt = kmblti_initcipher(salt, passwd, strlen(passwd), iv);
+    switch (fversion) {
+        case 0:
+            ctxt = kmblti_initcipher_v0(salt, passwd, strlen(passwd), iv);
+        case 1: /* Fall-through: */
+        default:
+            ctxt = kmblti_initcipher_v1(salt, passwd, strlen(passwd),
+                                        iterations, iv);
+            break;
+    }
     cnt = buffsz / 8;
     bptr = buff;
     while (cnt--) {
@@ -310,16 +389,17 @@ static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
 
         bptr += 8;
     }
-    wrcnt += fwrite((const void*)buff, buffsz, (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)buff, buffsz, fp_key);
     cm_bf_free(ctxt);
 
-    if (wrcnt != 5) {
+    if (wr_errs > 0 || ferror(fp_key) != 0) {
         fprintf(stderr, _("Failed to create new key file\n"));
         eflag = ERR_BADFILE;
         goto bail_out;
     }
 
   bail_out:
+
     if (buff != NULL) sec_free((void*)buff);
     if (passwd != NULL) sec_free((void*)passwd);
 
@@ -327,44 +407,88 @@ static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-static void *kmblti_md_prepare()
-{   cm_sha1_ctxt_t *mdcontext;
 
-    mdcontext = cm_sha1_init();
+/*
+ *  ==== Pure password based key-manager ====
+ */
 
-    return (void*)mdcontext;
+static int kmpswd_init_algs(void)
+{
+    /* Nothing needed */
+    return 0;
 }
 
 
-static void kmblti_md_block(void *state, unsigned char *buff, size_t len)
-{   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
-
-    /* Apply built-in SHA1 hash to data-block: */
-    cm_sha1_block(ctxt, buff, len);
+static int kmpswd_free_algs(void)
+{
+    /* Nothing needed */
+    return 0;
 }
 
 
-static void kmblti_md_final(void *state, unsigned char **mdval, size_t *mdlen)
-{   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
+static int kmpswd_bind(bound_tgtdefn_t *bound, FILE *fp_key)
+{   keyinfo_t *keyinfo = &bound->tgt->key;
+    int compat = 0;
 
-    cm_sha1_final(ctxt, mdval, mdlen);
+    if (keyinfo->format != NULL) {
+        compat = (strcmp(keyinfo->format, "password") == 0);
+    }
+
+    if (compat) {
+        if (keyinfo->digestalg == NULL) {
+            keyinfo->digestalg = cm_strdup("sha1");
+        }
+    }
+
+    return compat;
 }
 
 
-static void kmblti_md_release(void *state)
-{   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
-
-    cm_sha1_free(ctxt);
+static unsigned kmpswd_get_properties(const bound_tgtdefn_t *boundtgt)
+{
+    return (KM_PROP_HASPASSWD | KM_PROP_FIXEDLOC | KM_PROP_FORMATTED);
 }
 
 
-keymanager_t keymgr_blti = {
-    "builtin", 0,   kmblti_init_algs, kmblti_free_algs,
-                    kmblti_mk_default, kmblti_is_compat,
-		    kmblti_get_properties,
-                    kmblti_get_key, kmblti_put_key,
-                    kmblti_md_prepare, kmblti_md_block,
-                    kmblti_md_final, kmblti_md_release,
+static int kmpswd_get_key(bound_tgtdefn_t *boundtgt,
+            const km_pw_context_t *pw_ctxt,
+            unsigned char **key, int *keylen, FILE *fp_key)
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    char *passwd=NULL;
+    int eflag=ERR_NOERROR;
+    const unsigned iterations = 1 << 14;
+
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 0, 0);
+    if (eflag != ERR_NOERROR) goto bail_out;
+
+    *keylen = (keyinfo->maxlen >= 0 ? keyinfo->maxlen : 16);
+    cm_pwd_fortify(passwd, iterations, NULL, 16, key, (size_t)*keylen);
+
+    /* { size_t pos; fprintf(stderr,"pass-key: 0x"); for (pos=0; pos<*keylen; ++pos) fprintf(stderr,"%02x",(unsigned)(*key)[pos]); fprintf(stderr,"\n"); } */
+
+  bail_out:
+
+    if (passwd != NULL) sec_free((void*)passwd);
+
+    return eflag;
+}
+
+
+static int kmpswd_put_key(bound_tgtdefn_t *boundtgt,
+            const km_pw_context_t *pw_ctxt,
+            const unsigned char *key, const int keylen, FILE *fp_key)
+{
+    /* This operation isn't valid for a pure-password key */
+
+    return ERR_NOTSUPPORTED;
+}
+
+
+
+keymanager_t keymgr_pswd = {
+    "password", 0,  kmpswd_init_algs, kmpswd_free_algs,
+                    kmpswd_bind, kmpswd_get_properties,
+                    kmpswd_get_key, kmpswd_put_key,
     NULL
 #ifdef TESTING
     , NULL, NULL, 0
@@ -372,12 +496,24 @@ keymanager_t keymgr_blti = {
 };
 
 
+keymanager_t keymgr_blti = {
+    "builtin", 0,   kmblti_init_algs, kmblti_free_algs,
+                    kmblti_bind, kmblti_get_properties,
+                    kmblti_get_key, kmblti_put_key,
+    &keymgr_pswd
+#ifdef TESTING
+    , NULL, NULL, 0
+#endif
+};
+
 
 keymanager_t *kmblti_gethandle(void)
 {
     return &keymgr_blti;
 }
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  (C)Copyright 2007-2008, RW Penney
+ *  (C)Copyright 2007-2009, RW Penney
  */
