@@ -1,7 +1,7 @@
 /*
- *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  Methods for encryption/security mechanisms for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -55,7 +55,7 @@ cm_testinfo_t *test_ctxtptr;
 
 
 /*
- *  keyfile format is:
+ *  Keyfile format is:
  *      char magic[7]="cm-gcry";
  *      char version;
  *      uint16{LSB-first} cipher_blocklength, keylength;
@@ -169,7 +169,7 @@ static int kmgcry_get_algos(const keyinfo_t *keyinfo,
 
     *cipher = gcry_cipher_map_name(algstr);
     if (*cipher == 0) {
-        fprintf(stderr, _("couldn't find libgcrypt cipher \"%s\"\n"), algstr);
+        fprintf(stderr, _("Couldn't find libgcrypt cipher \"%s\"\n"), algstr);
         eflag = ERR_BADALGORITHM;
         goto bail_out;
     }
@@ -384,7 +384,10 @@ static int kmgcry_initcipher(int cipher, int ciphermode, int digest,
 
 static int kmgcry_init_algs()
 {
-    /* Nothing needed */
+    (void)gcry_check_version(NULL);     /* Initializes library as side-effect */
+
+    (void)gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+
     return 0;
 }
 
@@ -396,52 +399,28 @@ static int kmgcry_free_algs()
 }
 
 
-void kmgcry_mk_default(keyinfo_t *keyinfo)
-{
-    if (keyinfo == NULL) return;
-
-    if (keyinfo->digestalg == NULL) {
-        keyinfo->digestalg = cm_strdup("md5");
-    }
-
-    if (keyinfo->cipheralg == NULL) {
-        keyinfo->cipheralg = cm_strdup("blowfish");
-    }
-}
-
-
-static int kmgcry_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
-{   char buff[32];
-
-    if (keyinfo->format != NULL) {
-        return (strcmp(keyinfo->format, "libgcrypt") == 0);
-    } else {
-        if (fp_key != NULL) {
-            /* check Header of existing key-file: */
-            fread((void*)buff, kmgcry_maglen, (size_t)1, fp_key);
-            return (strncmp(buff, kmgcry_magstr, kmgcry_maglen) == 0);
-        }
-    }
-
-    return 0;
-}
-
-
-static int kmgcryossl_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
-    /* OpenSSL-compatibility version of kmgcy_is_compat */
-{   char buff[32];
+static int kmgcry_bind(bound_tgtdefn_t *bound, FILE *fp_key)
+{   keyinfo_t *keyinfo = &bound->tgt->key;
+    char buff[32];
     int compat = 0;
 
     if (keyinfo->format != NULL) {
-        compat |= (strcmp(keyinfo->format, "openssl-compat") == 0);
-#if !defined(HAVE_OPENSSL) || !HAVE_OPENSSL
-        compat |= (strcmp(keyinfo->format, "openssl") == 0);
-#endif
+        compat = (strcmp(keyinfo->format, "libgcrypt") == 0);
     } else {
         if (fp_key != NULL) {
             /* Check header of existing key-file: */
-            fread((void*)buff, kmgcryossl_maglen, (size_t)1, fp_key);
-            compat |= (strncmp(buff, kmgcryossl_magstr, kmgcryossl_maglen) == 0);
+            compat = (cm_fread((void*)buff, kmgcry_maglen, fp_key) == 0
+                        && strncmp(buff, kmgcry_magstr, kmgcry_maglen) == 0);
+        }
+    }
+
+    if (compat) {
+        if (keyinfo->digestalg == NULL) {
+            keyinfo->digestalg = cm_strdup("md5");
+        }
+
+        if (keyinfo->cipheralg == NULL) {
+            keyinfo->cipheralg = cm_strdup("blowfish");
         }
     }
 
@@ -449,23 +428,54 @@ static int kmgcryossl_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
 }
 
 
-static unsigned kmgcry_get_properties(const keyinfo_t *keyinf)
-{
-    return KM_PROP_HASPASSWD;
+static int kmgcryossl_bind(bound_tgtdefn_t *bound, FILE *fp_key)
+    /* OpenSSL-compatibility version of kmgcy_bind */
+{   keyinfo_t *keyinfo = &bound->tgt->key;
+    char buff[32];
+    int compat = 0;
+
+    if (keyinfo->format != NULL) {
+        compat |= (strcmp(keyinfo->format, "openssl-compat") == 0);
+        compat |= (strcmp(keyinfo->format, "openssl") == 0);
+    } else {
+        if (fp_key != NULL) {
+            /* Check header of existing key-file: */
+            compat = (cm_fread((void*)buff, kmgcryossl_maglen, fp_key) == 0
+                        && strncmp(buff, kmgcryossl_magstr, kmgcryossl_maglen) == 0);
+        }
+    }
+
+    if (compat) {
+        if (keyinfo->digestalg == NULL) {
+            keyinfo->digestalg = cm_strdup("md5");
+        }
+
+        if (keyinfo->cipheralg == NULL) {
+            keyinfo->cipheralg = cm_strdup("blowfish");
+        }
+    }
+
+    return compat;
 }
 
 
-static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
+static unsigned kmgcry_get_properties(const bound_tgtdefn_t *boundtgt)
+{
+    return (KM_PROP_HASPASSWD | KM_PROP_NEEDSKEYFILE);
+}
+
+
+static int kmgcry_get_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key,
-	    km_overrides_t *overrides)
+            unsigned char **key, int *keylen, FILE *fp_key)
     /* Extract key from libgcrypt-encrypted file */
-{   gcry_cipher_hd_t chd;
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char *hbuff=NULL, salt[kmgcry_saltlen], *buff=NULL, *bptr;
     size_t cblksz;
     uint32_t chksum, chksum0;
-    int cnt, cipher, ciphermode, digest, eflag=ERR_NOERROR;
+    int cnt, rd_errs=0, cipher, ciphermode, digest, eflag=ERR_NOERROR;
 
     *key = NULL; *keylen = 0;
     hbuff = sec_realloc(hbuff, (kmgcry_maglen + 4));
@@ -474,32 +484,32 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
     if (eflag != ERR_NOERROR) goto bail_out;
     gcry_cipher_algo_info(cipher, GCRYCTL_GET_BLKLEN, NULL, &cblksz);
 
-    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 0, 0);
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 0, 0);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     /* Read key header: */
-    fread((void*)hbuff, (size_t)1, kmgcry_maglen, fp_key);
+    rd_errs += cm_fread((void*)hbuff, kmgcry_maglen, fp_key);
     if (strncmp((const char*)hbuff, kmgcry_magstr, kmgcry_maglen) != 0) {
         fprintf(stderr, _("Bad keyfile format (libgcrypt)\n"));
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    fread((void*)hbuff, (size_t)1, (size_t)1, fp_key);
+    rd_errs += cm_fread((void*)hbuff, (size_t)1, fp_key);
     if (hbuff[0] != '\0') {
         fprintf(stderr, "Bad keyfile version [%d]\n", (int)buff[0]);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    fread((void*)hbuff, (size_t)4, (size_t)1, fp_key);
-    if (hbuff[0] != (cblksz & 0xff) || hbuff[1] != ((cblksz & 0xff00) >> 8)) {
+    rd_errs += cm_fread((void*)hbuff, (size_t)4, fp_key);
+    if (pack_uint16(hbuff) != cblksz) {
         fprintf(stderr, "Mismatched cipher block size\n");
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    *keylen = ((unsigned)hbuff[2]) | (((unsigned)hbuff[3]) << 8);
+    *keylen = pack_uint16(hbuff + 2);
 
     /* Read salt from keyfile: */
-    fread((void*)salt, (size_t)1, sizeof(salt), fp_key);
+    rd_errs += cm_fread((void*)salt, sizeof(salt), fp_key);
 
     /* Read encrypted key from keyfile: */
     eflag = kmgcry_initcipher(cipher, ciphermode, digest,
@@ -509,7 +519,7 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
     buff = sec_realloc(buff, cnt * cblksz);
     bptr = buff;
     while (cnt--) {
-        fread((void*)bptr, cblksz, (size_t)1, fp_key);
+        rd_errs += cm_fread((void*)bptr, cblksz, fp_key);
         gcry_cipher_decrypt(chd, (void*)bptr, cblksz, NULL, 0);
         bptr += cblksz;
     }
@@ -523,7 +533,7 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
                 break;
             case 1:     /* fall through... */
             default:
-                fprintf(stderr, _("checksum mismatch in keyfile (gcry, %x != %x)\n"),
+                fprintf(stderr, _("Checksum mismatch in keyfile (gcry, %x != %x)\n"),
                         (unsigned)chksum, (unsigned)chksum0);
                 break;
         }
@@ -536,7 +546,7 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
     *key = (unsigned char*)sec_realloc((void*)*key, (size_t)*keylen);
     memcpy(*key, buff, (size_t)*keylen);
 
-    if (ferror(fp_key) != 0) {
+    if (rd_errs > 0 || ferror(fp_key) != 0) {
         fprintf(stderr, _("Key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
         eflag = ERR_BADFILE;
@@ -552,35 +562,34 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmgcry_put_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key,
-	    const km_overrides_t *overrides)
+            const unsigned char *key, const int keylen, FILE *fp_key)
     /* Store key in libgcrypt-encrypted file */
-{   gcry_cipher_hd_t chd;
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char hbuff[4], salt[kmgcry_saltlen], *buff=NULL, *bptr;
-    size_t buffsz, cblksz, wrcnt;
-    int cnt, cipher, ciphermode, digest, eflag=ERR_NOERROR;
+    size_t buffsz, cblksz;
+    int cnt, wr_errs = 0, cipher, ciphermode, digest, eflag=ERR_NOERROR;
 
     eflag = kmgcry_get_algos(keyinfo, &cipher, &ciphermode, &digest);
     if (eflag != ERR_NOERROR) goto bail_out;
     gcry_cipher_algo_info(cipher, GCRYCTL_GET_BLKLEN, NULL, &cblksz);
 
-    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 1, 1);
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 1, 1);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     /* Write key header: */
-    wrcnt = 0;
-    wrcnt += fwrite((const void*)kmgcry_magstr, kmgcry_maglen, (size_t)1, fp_key);
-    wrcnt += fwrite((const void*)&kmgcry_version, (size_t)1, (size_t)1, fp_key);
-    hbuff[0] = (cblksz & 0xff); hbuff[1] = (cblksz & 0xff00) >> 8;
-    hbuff[2] = (keylen & 0xff); hbuff[3] = (keylen & 0xff00) >> 8;
-    wrcnt += fwrite((const void*)hbuff, (size_t)4, (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)kmgcry_magstr, kmgcry_maglen, fp_key);
+    wr_errs += cm_fwrite((const void*)&kmgcry_version, (size_t)1, fp_key);
+    unpack_uint16(hbuff, (uint16_t)cblksz);
+    unpack_uint16(hbuff + 2, (uint16_t)keylen);
+    wr_errs += cm_fwrite((const void*)hbuff, (size_t)4, fp_key);
 
     /* Generate salt & record in keyfile: */
     get_randkey(salt, sizeof(salt));
-    wrcnt += fwrite((const void*)salt, sizeof(salt), (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)salt, sizeof(salt), fp_key);
 
     /* Augment key with simple checksum: */
     buff = km_aug_key(key, (unsigned)keylen, (unsigned)cblksz, &buffsz);
@@ -593,19 +602,19 @@ static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
     bptr = buff;
     while (cnt--) {
         gcry_cipher_encrypt(chd, (void*)bptr, cblksz, NULL, 0);
-        wrcnt += fwrite((const void*)bptr, cblksz, (size_t)1, fp_key);
+        wr_errs += cm_fwrite((const void*)bptr, cblksz, fp_key);
         bptr += cblksz;
     }
     gcry_cipher_close(chd);
 
-    if (wrcnt != (4 + buffsz / cblksz)) {
+    if (wr_errs > 0 || ferror(fp_key) != 0) {
         fprintf(stderr, _("Failed to create new key file\n"));
-        fprintf(stderr, "wrcnt=%d\n", (int)wrcnt);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
 
   bail_out:
+
     if (buff != NULL) sec_free((void*)buff);
     if (passwd != NULL) sec_free((void*)passwd);
 
@@ -615,16 +624,17 @@ static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
 
 #if USE_GCRYOSSL
 
-static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmgcryossl_get_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key,
-	    km_overrides_t *overrides)
+            unsigned char **key, int *keylen, FILE *fp_key)
     /* Extract key from OpenSSL-compatible file via libgcrypt */
-{   gcry_cipher_hd_t chd;
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char *hbuff=NULL, salt[kmgcryossl_saltlen], *buff=NULL;
     size_t cblksz, buffsz=0, pos, ofs, idx;
-    int kbad=0, cipher, ciphermode, digest, eflag=ERR_NOERROR;
+    int kbad=0, cipher, ciphermode, digest,
+        rd_errs=0, eflag=ERR_NOERROR;
 
     *key = NULL; *keylen = 0;
     hbuff = sec_realloc(hbuff, kmgcryossl_maglen);
@@ -633,19 +643,19 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
     if (eflag != ERR_NOERROR) goto bail_out;
     gcry_cipher_algo_info(cipher, GCRYCTL_GET_BLKLEN, NULL, &cblksz);
 
-    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 0, 0);
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 0, 0);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     /* Read key header: */
-    fread((void*)hbuff, (size_t)1, kmgcryossl_maglen, fp_key);
+    rd_errs += cm_fread((void*)hbuff, kmgcryossl_maglen, fp_key);
     if (strncmp((const char*)hbuff, kmgcryossl_magstr, kmgcryossl_maglen) != 0) {
-        fprintf(stderr, _("bad keyfile format (openssl-compat)\n"));
+        fprintf(stderr, _("Bad keyfile format (openssl-compat)\n"));
         eflag = ERR_BADFILE;
         goto bail_out;
     }
 
     /* Read salt from keyfile: */
-    fread((void*)salt, (size_t)1, sizeof(salt), fp_key);
+    rd_errs += cm_fread((void*)salt, sizeof(salt), fp_key);
 
     /* read encrypted key from keyfile: */
     eflag = kmgcry_initcipher(cipher, ciphermode, digest,
@@ -657,7 +667,7 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
             buffsz = (buffsz * 2) + 4 * cblksz;
             buff = sec_realloc(buff, buffsz);
         }
-        if (fread((void*)(buff + pos), cblksz, (size_t)1, fp_key) != 1) break;
+        if (cm_fread((void*)(buff + pos), cblksz, fp_key) != 0) break;
         gcry_cipher_decrypt(chd, (void*)(buff + pos), cblksz, NULL, 0);
         pos += cblksz;
     }
@@ -692,7 +702,7 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
     *key = (unsigned char*)sec_realloc((void*)*key, (size_t)*keylen);
     memcpy(*key, buff, (size_t)*keylen);
 
-    if (ferror(fp_key) != 0) {
+    if (rd_errs > 0 || ferror(fp_key) != 0) {
         fprintf(stderr, _("Key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
         eflag = ERR_BADFILE;
@@ -708,31 +718,30 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-static int kmgcryossl_put_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmgcryossl_put_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key,
-	    const km_overrides_t *overrides)
+            const unsigned char *key, const int keylen, FILE *fp_key)
     /* Store key in OpenSSL-compatible file via libgcrypt */
-{   gcry_cipher_hd_t chd;
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char salt[kmgcryossl_saltlen], *buff=NULL;
-    size_t buffsz, cblksz, pos, wrcnt;
-    int cipher, ciphermode, digest, eflag=ERR_NOERROR;
+    size_t buffsz, cblksz, pos;
+    int wr_errs=0, cipher, ciphermode, digest, eflag=ERR_NOERROR;
 
     eflag = kmgcry_get_algos(keyinfo, &cipher, &ciphermode, &digest);
     if (eflag != ERR_NOERROR) goto bail_out;
     gcry_cipher_algo_info(cipher, GCRYCTL_GET_BLKLEN, NULL, &cblksz);
 
-    eflag = km_get_passwd(ident, pw_ctxt, &passwd, 1, 1);
+    eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 1, 1);
     if (eflag != ERR_NOERROR) goto bail_out;
 
     /* Write key header: */
-    wrcnt = 0;
-    wrcnt += fwrite((const void*)kmgcryossl_magstr, kmgcryossl_maglen, (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)kmgcryossl_magstr, kmgcryossl_maglen, fp_key);
 
     /* Generate salt & record in keyfile: */
     get_randkey(salt, sizeof(salt));
-    wrcnt += fwrite((const void*)salt, sizeof(salt), (size_t)1, fp_key);
+    wr_errs += cm_fwrite((const void*)salt, sizeof(salt), fp_key);
 
     /* Pad key-data with end-marker: */
     buffsz = cblksz * ((keylen + cblksz) / cblksz);
@@ -747,14 +756,13 @@ static int kmgcryossl_put_key(const char *ident, const keyinfo_t *keyinfo,
     pos = 0;
     while (pos < buffsz) {
         gcry_cipher_encrypt(chd, (void*)(buff + pos), cblksz, NULL, 0);
-        wrcnt += fwrite((const void*)(buff + pos), cblksz, (size_t)1, fp_key);
+        wr_errs += cm_fwrite((const void*)(buff + pos), cblksz, fp_key);
         pos += cblksz;
     }
     gcry_cipher_close(chd);
 
-    if (wrcnt != (2 + buffsz / cblksz)) {
+    if (wr_errs > 0 || ferror(fp_key) != 0) {
         fprintf(stderr, _("Failed to create new key file\n"));
-        fprintf(stderr, "wrcnt=%d\n", (int)wrcnt);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
@@ -769,49 +777,11 @@ static int kmgcryossl_put_key(const char *ident, const keyinfo_t *keyinfo,
 #endif  /* USE_GCRYOSSL */
 
 
-static void *kmgcry_md_prepare(void)
-{   gcry_md_hd_t *handle;
-
-    handle = (gcry_md_hd_t*)malloc(sizeof(gcry_md_hd_t));
-    (void)gcry_md_open(handle, GCRY_MD_SHA1, 0);
-
-    return (void*)handle;
-}
-
-
-static void kmgcry_md_block(void *state, unsigned char *buff, size_t len)
-{   gcry_md_hd_t *handle=(gcry_md_hd_t*)state;
-
-    gcry_md_write(*handle, (const void*)buff, len);
-}
-
-
-static void kmgcry_md_final(void *state, unsigned char **mdval, size_t *mdlen)
-{   gcry_md_hd_t *handle=(gcry_md_hd_t*)state;
-    unsigned char *buff;
-    int algo;
-
-    gcry_md_final(*handle);
-    algo = gcry_md_get_algo(*handle);
-    *mdlen = gcry_md_get_algo_dlen(algo);
-
-    buff = gcry_md_read(*handle, algo);
-    *mdval = (unsigned char*)malloc(*mdlen);
-    memcpy((void*)*mdval, (const void*)buff, *mdlen);
-}
-
-
-static void kmgcry_md_release(void *state)
-{   gcry_md_hd_t *handle=(gcry_md_hd_t*)state;
-
-    gcry_md_close(*handle);
-    free((void*)handle);
-}
-
 #  ifdef TESTING
 
 static int kmgcry_test_hash()
-{   void *mdcontext;
+{   gcry_md_hd_t mdcontext;
+    int algo;
     unsigned char *mdval=NULL;
     size_t mdlen, i;
     unsigned q;
@@ -820,10 +790,12 @@ static int kmgcry_test_hash()
 
     CM_TEST_START("libgcrypt hashing");
 
-    mdcontext = kmgcry_md_prepare();
-    CM_ASSERT_DIFFERENT(NULL, mdcontext);
-    kmgcry_md_block(mdcontext, (unsigned char*)str, strlen(str));
-    kmgcry_md_final(mdcontext, &mdval, &mdlen);
+    gcry_md_open(&mdcontext, GCRY_MD_SHA1, 0);
+    gcry_md_write(mdcontext, (const void*)str, strlen(str));
+    gcry_md_final(mdcontext);
+    algo = gcry_md_get_algo(mdcontext);
+    mdlen = gcry_md_get_algo_dlen(algo);
+    mdval = gcry_md_read(mdcontext, algo);
     CM_ASSERT_DIFFERENT(NULL, mdval);
     CM_ASSERT_EQUAL(strlen(hash)/2, mdlen);
     for (i=0; i<mdlen; ++i) {
@@ -831,7 +803,7 @@ static int kmgcry_test_hash()
         CM_ASSERT_EQUAL(q, (unsigned)mdval[i]);
     }
 
-    kmgcry_md_release(mdcontext);
+    gcry_md_close(mdcontext);
 
     CM_TEST_OK();
 }
@@ -842,11 +814,12 @@ static void kmgcry_testctxt(cm_testinfo_t *context)
 }
 
 static int kmgcry_runtests()
-{
-    kmgcry_test_hash();
-    kmgcry_test_getalgos();
+{   int flg = 0;
 
-    return 0;
+    flg |= kmgcry_test_hash();
+    flg |= kmgcry_test_getalgos();
+
+    return flg;
 }
 
 #  endif    /* TESTING */
@@ -856,11 +829,8 @@ static int kmgcry_runtests()
 
 keymanager_t keymgr_gcryossl = {
     "openssl-compat", 0,    kmgcry_init_algs, kmgcry_free_algs,
-                      kmgcry_mk_default, kmgcryossl_is_compat,
-                      kmgcry_get_properties,
+                      kmgcryossl_bind, kmgcry_get_properties,
                       kmgcryossl_get_key, kmgcryossl_put_key,
-                      kmgcry_md_prepare, kmgcry_md_block,
-                      kmgcry_md_final, kmgcry_md_release,
     NULL
 #ifdef TESTING
     , NULL, NULL, 0
@@ -871,11 +841,8 @@ keymanager_t keymgr_gcryossl = {
 
 keymanager_t keymgr_gcry = {
     "libgcrypt", 0,   kmgcry_init_algs, kmgcry_free_algs,
-                      kmgcry_mk_default, kmgcry_is_compat,
-                      kmgcry_get_properties,
+                      kmgcry_bind, kmgcry_get_properties,
                       kmgcry_get_key, kmgcry_put_key,
-                      kmgcry_md_prepare, kmgcry_md_block,
-                      kmgcry_md_final, kmgcry_md_release,
 #if USE_GCRYOSSL
     &keymgr_gcryossl
 #else
@@ -893,7 +860,7 @@ keymanager_t keymgr_gcry = {
 #ifndef AS_MODULE
 
 #if defined(TESTING)
-#  define MOD_PATH "./cm-gcry.so"
+#  define MOD_PATH CM_SRCDIR "/cm-gcry.so"
 #else
 #  define MOD_PATH CM_MODULE_DIR "/cm-gcry.so"
 #endif
@@ -913,6 +880,8 @@ keymanager_t *kmgcry_gethandle()
 
 #endif  /* !AS_MODULE */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
