@@ -1,7 +1,7 @@
 /*
- *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  Methods for encryption/security mechanisms for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -42,14 +42,20 @@
 
 #include "armour.h"
 #include "cryptmount.h"
+#include "tables.h"
 #include "utils.h"
 #ifdef TESTING
+#  include <dirent.h>
 #  include "cmtesting.h"
 #endif
 
 
 #ifndef AS_MODULE
 
+keymanager_t *kmblti_gethandle(void);
+keymanager_t *kmgcry_gethandle(void);
+keymanager_t *kmluks_gethandle(void);
+keymanager_t *init_keymanager(keymanager_t *km);
 
 /* List of all available key-managers (to be constructed later): */
 static keymanager_t *keymgrs = NULL;
@@ -71,46 +77,54 @@ static int kmraw_free_algs(void)
 }
 
 
-static void kmraw_mk_default(keyinfo_t *keyinfo)
-{
-    if (keyinfo == NULL) return;
+static int kmraw_bind(bound_tgtdefn_t *bound, FILE *fp_key)
+{   keyinfo_t *keyinfo = &bound->tgt->key;
+    int compat = 0;
 
-    if (keyinfo->digestalg == NULL) {
-        keyinfo->digestalg = cm_strdup("none");
-    }
-
-    if (keyinfo->cipheralg == NULL) {
-        keyinfo->cipheralg = cm_strdup("none");
-    }
-}
-
-
-static int kmraw_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
-{
     if (keyinfo->format != NULL) {
-        return (strcmp(keyinfo->format, "raw") == 0);
+        compat = (strcmp(keyinfo->format, "raw") == 0);
     } else {
         if (keyinfo->cipheralg != NULL) {
             return (strcmp(keyinfo->cipheralg, "none") == 0);
         }
     }
 
-    return 0;
+    if (compat) {
+        if (keyinfo->digestalg == NULL) {
+            keyinfo->digestalg = cm_strdup("none");
+        }
+
+        if (keyinfo->cipheralg == NULL) {
+            keyinfo->cipheralg = cm_strdup("none");
+        }
+    }
+
+    return compat;
 }
 
 
-static unsigned kmraw_get_properties(const keyinfo_t *keyinf)
-{
-    return 0;
+static unsigned kmraw_get_properties(const bound_tgtdefn_t *boundtgt)
+{   struct stat sbuff;
+    unsigned props;
+
+    /* We must flag that we have no password & have fixed location,
+       otherwise keyfile=/dev/random could be renamed on password-changing! */
+    props = KM_PROP_NEEDSKEYFILE | KM_PROP_FIXEDLOC;
+
+    if (stat(boundtgt->tgt->key.filename, &sbuff) == 0) {
+        props |= KM_PROP_FORMATTED;
+    }
+
+    return props;
 }
 
 
-static int kmraw_get_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmraw_get_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key,
-            km_overrides_t *overrides)
+            unsigned char **key, int *keylen, FILE *fp_key)
     /* Extract key from unencrypted (plain) file */
-{   enum { BUFFSZ=512 };
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    enum { BUFFSZ=512 };
     char buff[BUFFSZ];
     size_t len, lmt;
     int eflag=ERR_NOERROR;
@@ -122,7 +136,6 @@ static int kmraw_get_key(const char *ident, const keyinfo_t *keyinfo,
         goto bail_out;
     }
 
-
     /* Read data directly from keyfile: */
     for (;;) {
         lmt = (keyinfo->maxlen > 0 && (*keylen + BUFFSZ) > keyinfo->maxlen
@@ -149,15 +162,15 @@ static int kmraw_get_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-static int kmraw_put_key(const char *ident, const keyinfo_t *keyinfo,
+static int kmraw_put_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key,
-            const km_overrides_t *overrides)
+            const unsigned char *key, const int keylen, FILE *fp_key)
     /* Store key in unencrypted (plain) file */
-{   int eflag=ERR_NOERROR;
+{   const keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    int eflag=ERR_NOERROR;
 
     /* Write data directly into keyfile: */
-    if (fwrite((const void*)key, (size_t)keylen, (size_t)1, fp_key) == 0
+    if (cm_fwrite((const void*)key, (size_t)keylen, fp_key) != 0
       || ferror(fp_key) != 0) {
         fprintf(stderr, _("Key-writing failed for \"%s\"\n"),
                 keyinfo->filename);
@@ -168,44 +181,10 @@ static int kmraw_put_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-static void *kmraw_md_prepare(void)
-{   cm_sha1_ctxt_t *mdcontext;
-
-    mdcontext = cm_sha1_init();
-
-    return (void*)mdcontext;
-}
-
-
-static void kmraw_md_block(void *state, unsigned char *buff, size_t len)
-{   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
-
-    /* Apply built-in SHA1 hash to data-block: */
-    cm_sha1_block(ctxt, buff, len);
-}
-
-
-static void kmraw_md_final(void *state, unsigned char **mdval, size_t *mdlen)
-{   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
-
-    cm_sha1_final(ctxt, mdval, mdlen);
-}
-
-
-static void kmraw_md_release(void *state)
-{   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
-
-    cm_sha1_free(ctxt);
-}
-
-
 static keymanager_t keymgr_raw = {
     "raw", 0,       kmraw_init_algs, kmraw_free_algs,
-                    kmraw_mk_default, kmraw_is_compat,
-                    kmraw_get_properties,
+                    kmraw_bind, kmraw_get_properties,
                     kmraw_get_key, kmraw_put_key,
-                    kmraw_md_prepare, kmraw_md_block,
-                    kmraw_md_final, kmraw_md_release,
     NULL
 #ifdef TESTING
     , NULL, NULL, 0
@@ -217,19 +196,19 @@ static keymanager_t keymgr_raw = {
 
 
 /*
- *  ==== abstract key-management interfaces ====
+ *  ==== Abstract key-management interfaces ====
  */
 
 #ifndef AS_MODULE
 
 static keymanager_t **add_keymgr(keymanager_t **ptr, keymanager_t *km)
-    /* add key-manager interface definition(s) to list of available managers */
+    /* Add key-manager interface definition(s) to list of available managers */
 {
     if (km == NULL) return ptr;
 
     *ptr = km;
 
-    /* allow for addend itself being a list of key-managers: */
+    /* Allow for addend itself being a list of key-managers: */
     for (;;) {
 #ifdef TESTING
         if (km->install_testctxt != NULL) km->install_testctxt(test_ctxtptr);
@@ -243,7 +222,7 @@ static keymanager_t **add_keymgr(keymanager_t **ptr, keymanager_t *km)
 
 
 static void build_keymgrs(void)
-    /* construct list of available key-managers */
+    /* Construct list of available key-managers */
 {   keymanager_t **ptr;
 
     if (keymgrs != NULL) return;    /* already initialized */
@@ -257,7 +236,7 @@ static void build_keymgrs(void)
 
 
 const char **get_keymgr_list(void)
-    /* construct list of key-manager names */
+    /* Construct list of key-manager names */
 {   keymanager_t *km;
     const char **arr=NULL;
     int i, cnt;
@@ -275,30 +254,16 @@ const char **get_keymgr_list(void)
 }
 
 
-keymanager_t *get_keymanager(const struct keyinfo *keyinfo, FILE *fp_key)
-    /* Search for keymanager that supports given key */
-{   keymanager_t *km;
-
-    if (keyinfo == NULL) return NULL;
-    build_keymgrs();
-
-    for (km=keymgrs; km!=NULL; km=km->next) {
-        if (fp_key != NULL) (void)fseek(fp_key, 0L, SEEK_SET);
-        if (km->is_compat(keyinfo, fp_key)) {
-            return init_keymanager(km);
-        }
-    }
-
-    return NULL;
-}
-
-
 keymanager_t *init_keymanager(keymanager_t *km)
 {
     if (km != NULL) {
         if (!km->initialized) {
-            km->init_algs();    /* FIXME - check return status */
-            km->initialized = 1;
+            if (km->init_algs() == 0) {
+                km->initialized = 1;
+            } else {
+                fprintf(stderr, "Failed to initialize keymanager \"%s\"\n",
+                        km->ident);
+            }
         }
     }
 
@@ -319,46 +284,100 @@ int free_keymanagers(void)
     return 0;
 }
 
+
+bound_tgtdefn_t *alloc_boundtgt(const tgtdefn_t *tgt)
+{   bound_tgtdefn_t *bound;
+
+    bound = (bound_tgtdefn_t*)malloc(sizeof(bound_tgtdefn_t));
+    bound->tgt = clone_tgtdefn(tgt);
+    bound->keymgr = NULL;
+    bound->km_data = NULL;
+
+    return bound;
+}
+
+
+void free_boundtgt(bound_tgtdefn_t *boundtgt)
+{
+    if (boundtgt == NULL) return;
+
+    free_tgtdefn(boundtgt->tgt);
+
+    if (boundtgt->km_data != NULL) free((void*)boundtgt->km_data);
+
+    free((void*)boundtgt);
+}
+
+
+bound_tgtdefn_t *bind_tgtdefn(const tgtdefn_t *tgt)
+    /* Find keymanager that is able to handle given target & keyfile */
+{   bound_tgtdefn_t *bound;
+    keymanager_t *km;
+    FILE *fp_key=NULL;
+
+    if (tgt == NULL) return NULL;
+
+    build_keymgrs();
+    bound = alloc_boundtgt(tgt);
+    if (tgt->key.filename != NULL) {
+        fp_key = fopen(tgt->key.filename, "rb");
+    }
+
+    km = keymgrs;
+    while (km != NULL) {
+        if (fp_key != NULL) (void)fseek(fp_key, 0L, SEEK_SET);
+        if (km->bind(bound, fp_key)) {
+            bound->keymgr = init_keymanager(km);
+            break;
+        }
+        km = km->next;
+    }
+
+    if (bound->keymgr == NULL) {
+        free_boundtgt(bound);
+        bound = NULL;
+    }
+
+    if (fp_key != NULL) fclose(fp_key);
+
+    return bound;
+}
+
 #endif  /* !AS_MODULE */
 
 
-int cm_get_key(const char *ident, const keyinfo_t *keyinfo,
-            const km_pw_context_t *pw_ctxt, unsigned char **key, int *keylen,
-            km_overrides_t *overrides)
-{   keymanager_t *km;
-    FILE *fp=NULL;
-    unsigned attempts=0;
+int cm_get_key(bound_tgtdefn_t *boundtgt, const km_pw_context_t *pw_ctxt,
+                unsigned char **key, int *keylen)
+{   keyinfo_t *keyinfo = &boundtgt->tgt->key;
+    FILE *fp = NULL;
+    unsigned props, attempts = 0;
     int eflag=ERR_NOERROR;
 
-    if (keyinfo == NULL || keyinfo->filename == NULL) {
-        fprintf(stderr, _("Missing key-file for target \"%s\"\n"), ident);
-        eflag = ERR_BADFILE;
-        goto bail_out;
-    }
-
-    fp = fopen(keyinfo->filename, "rb");
-    if (fp == NULL) {
-        fprintf(stderr,
-            _("Failed to open keyfile \"%s\" for target \"%s\"\n"),
-            keyinfo->filename, ident);
-        eflag = ERR_BADFILE;
-        goto bail_out;
-    }
-
-    if ((km = get_keymanager(keyinfo, fp)) != NULL) {
-        do {
-            (void)fseek(fp, 0L, SEEK_SET);
-            eflag = km->get_key(ident, keyinfo, pw_ctxt, key, keylen,
-                                fp, overrides);
-            if (eflag == ERR_BADDECRYPT) sleep(1);
-        } while (++attempts < keyinfo->retries && eflag == ERR_BADDECRYPT);
+    if (keyinfo->filename != NULL) {
+        fp = fopen(keyinfo->filename, "rb");
+        if (fp == NULL) {
+            fprintf(stderr,
+                _("Failed to open keyfile \"%s\" for target \"%s\"\n"),
+                keyinfo->filename, boundtgt->tgt->ident);
+            eflag = ERR_BADFILE;
+            goto bail_out;
+        }
     } else {
-        fprintf(stderr,
-                _("Unrecognized key format (%s) for target \"%s\"\n"),
-                (keyinfo->format != NULL ? keyinfo->format : "NULL"), ident);
-        eflag = ERR_BADKEYFORMAT;
+        props = boundtgt->keymgr->get_properties(boundtgt);
+        if ((props & KM_PROP_NEEDSKEYFILE) != 0) {
+            fprintf(stderr, _("Missing keyfile for target \"%s\"\n"),
+                boundtgt->tgt->ident);
+            eflag = ERR_BADFILE;
+            goto bail_out;
+        }
     }
 
+    do {
+        if (fp != NULL) (void)fseek(fp, 0L, SEEK_SET);
+        eflag = boundtgt->keymgr->get_key(boundtgt, pw_ctxt, key, keylen, fp);
+        if (eflag == ERR_BADDECRYPT) sleep(1);
+    } while (++attempts < keyinfo->retries && eflag == ERR_BADDECRYPT);
+
   bail_out:
     if (fp != NULL) fclose(fp);
 
@@ -366,39 +385,30 @@ int cm_get_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-int cm_put_key(const char *ident, const keyinfo_t *keyinfo,
-            const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key,
-            const km_overrides_t *overrides)
-{   keymanager_t *km;
+int cm_put_key(bound_tgtdefn_t *boundtgt, const km_pw_context_t *pw_ctxt,
+            const unsigned char *key, const int keylen, FILE *fp_key)
+{   unsigned props;
     int eflag = ERR_NOERROR;
 
-    if (fp_key == NULL) {
-        fprintf(stderr, _("Missing output stream for target \"%s\"\n"), ident);
+    props = boundtgt->keymgr->get_properties(boundtgt);
+    if ((props & KM_PROP_NEEDSKEYFILE) != 0 && fp_key == NULL) {
+        fprintf(stderr, _("Missing output keyfile for target \"%s\"\n"),
+                boundtgt->tgt->ident);
         return ERR_BADFILE;
     }
 
-    if ((km = get_keymanager(keyinfo, NULL)) != NULL) {
-        eflag = km->put_key(ident, keyinfo, pw_ctxt, key, keylen,
-                            fp_key, overrides);
-    } else {
-        fprintf(stderr,
-                _("Unrecognized key format (%s) for target \"%s\"\n"),
-                (keyinfo->format != NULL ? keyinfo->format : "NULL"), ident);
-        eflag = ERR_BADKEYFORMAT;
-    }
+    eflag = boundtgt->keymgr->put_key(boundtgt, pw_ctxt, key, keylen, fp_key);
 
     return eflag;
 }
 
 
-unsigned cm_get_keyproperties(const keyinfo_t *keyinfo)
+unsigned cm_get_keyproperties(const bound_tgtdefn_t *boundtgt)
     /* Extract information about key, e.g. whether encrypted */
-{   keymanager_t *km;
-    unsigned props;
+{   unsigned props;
 
-    if ((km = get_keymanager(keyinfo, NULL)) != NULL) {
-        props = km->get_properties(keyinfo);
+    if (boundtgt != NULL && boundtgt->keymgr != NULL) {
+        props = boundtgt->keymgr->get_properties(boundtgt);
     } else {
         /* Safest to assume that key shouldn't be overwritten: */
         props = KM_PROP_FIXEDLOC | KM_PROP_FORMATTED;
@@ -408,58 +418,12 @@ unsigned cm_get_keyproperties(const keyinfo_t *keyinfo)
 }
 
 
-km_overrides_t *alloc_overrides()
-{   km_overrides_t *overrides;
-
-    overrides = (km_overrides_t*)malloc(sizeof(km_overrides_t));
-
-    overrides->mask = 0;
-    overrides->cipher = NULL;
-    overrides->km_data = NULL;
-
-    return overrides;
-}
-
-void merge_overrides(const cment_t *ent, const km_overrides_t *overrides,
-                    cment_t *merged)
-    /* Combine parameters suggested by key-manager with cment_t */
-{   int64_t delta;
-
-    if (ent == NULL || overrides == NULL) return;
-
-    memcpy((void*)merged, (const void*)ent, sizeof(cment_t));
-
-    if ((overrides->mask & KM_OR_START) != 0) {
-        delta = (overrides->start - ent->start);
-        if (delta >= 0) {
-            merged->start = overrides->start;
-            if (merged->length >= 0) merged->length -= delta;
-        }
-    }
-
-    if ((overrides->mask & KM_OR_CIPHER) != 0) {
-        merged->cipher = overrides->cipher;
-    }
-
-    if ((overrides->mask & KM_OR_IVOFFSET) != 0) {
-        merged->ivoffset = overrides->ivoffset;
-    }
-}
-
-
-void free_overrides(km_overrides_t *overrides)
-{
-    if (overrides == NULL) return;
-
-    if (overrides->cipher != NULL) free((void*)overrides->cipher);
-    if (overrides->km_data != NULL) free((void*)overrides->km_data);
-
-    free((void*)overrides);
-}
-
 
 #ifdef TESTING
 
+const char *km_legacy_key = "cryptmount012345";
+const char *km_legacy_passwd = "nothing";
+
 int km_test_managers(void)
 {   keymanager_t *km;
     int flg = 0;
@@ -470,36 +434,40 @@ int km_test_managers(void)
         if (km->run_tests != NULL) flg |= km->run_tests();
     }
 
+#if 0
+    km_archive_keys();
+#endif
+
     return flg;
 }
 
 
 int km_test_keyrw(void)
-    /* test key read-writing (creation/extraction) */
+    /* Test key read-writing (creation/extraction) */
 {   enum { MAXKEY=256 };
-    keymanager_t *km;
-    keyinfo_t keyinfo;
-    int i, keylen=0, keylen1;
+    const keymanager_t *km;
+    tgtdefn_t *target;
+    bound_tgtdefn_t *bound=NULL;
+    int i, keylen=0, keylen1, eflag;
     char str[256];
-    unsigned char key0[MAXKEY],*key1=NULL;
+    unsigned char key0[MAXKEY], *key1=NULL;
     FILE *fp;
-    km_overrides_t *overrides=NULL;
     extern cm_testinfo_t *test_ctxtptr;
 
-    CM_TEST_START("key read-write");
+    CM_TEST_START("Key read-write");
 
     build_keymgrs();
     for (km=keymgrs; km!=NULL; km=km->next) {
         if ((km->test_flags & CM_READONLY) != 0) continue;
-
-        keyinfo.format = NULL;
-        keyinfo.filename = "NOWHERE";
-        keyinfo.digestalg = NULL;
-        keyinfo.cipheralg = NULL;
-        keyinfo.maxlen = -1;
-        km->mk_default(&keyinfo);
         km->init_algs();
 
+        target = alloc_tgtdefn(NULL);
+        target->key.format = cm_strdup(km->ident);
+        target->key.filename = cm_strdup("NOWHERE");
+        target->key.digestalg = NULL;
+        target->key.cipheralg = NULL;
+        target->key.maxlen = -1;
+
         for (keylen=1; keylen<=MAXKEY; keylen<<=2) {
             sprintf(str, "Key read-write, %s, keylen=%d",
                 km->ident, keylen);
@@ -513,45 +481,172 @@ int km_test_keyrw(void)
             /* Write key to file: */
             fp = tmpfile();
             if (fp == NULL) CM_TEST_ABORT(context);
-            overrides = alloc_overrides();
-            i = km->put_key("TEST-OUT", &keyinfo, NULL,
-                            key0, keylen, fp, overrides);
-            CM_ASSERT_EQUAL(ERR_NOERROR, i);
+            bound = alloc_boundtgt(target);
+            if (!km->bind(bound, fp)) CM_TEST_FAIL();
+            eflag = km->put_key(bound, NULL, key0, keylen, fp);
+            if (eflag != ERR_NOTSUPPORTED) {
+                CM_ASSERT_EQUAL(ERR_NOERROR, eflag);
 
-            key1 = NULL; keylen1 = -keylen;
+                key1 = NULL; keylen1 = -keylen;
 
-            /* Try reading key back from file: */
-            rewind(fp);
-            i = km->get_key("TEST-IN", &keyinfo, NULL,
-                            &key1, &keylen1, fp, overrides);
-            CM_ASSERT_EQUAL(ERR_NOERROR, i);
-            CM_ASSERT_EQUAL(keylen, keylen1);
-            CM_ASSERT_DIFFERENT(key0, key1);
-            CM_ASSERT_DIFFERENT(NULL, key1);
-            for (i=0; i<keylen; ++i) {
-                CM_ASSERT_EQUAL(key0[i], key1[i]);
+                /* Try reading key back from file: */
+                rewind(fp);
+                eflag = km->get_key(bound, NULL, &key1, &keylen1, fp);
+                CM_ASSERT_EQUAL(ERR_NOERROR, eflag);
+                CM_ASSERT_EQUAL(keylen, keylen1);
+                CM_ASSERT_DIFFERENT(key0, key1);
+                CM_ASSERT_DIFFERENT(NULL, key1);
+                for (i=0; i<keylen; ++i) {
+                    CM_ASSERT_EQUAL(key0[i], key1[i]);
+                }
             }
 
-            free_overrides(overrides);
-            fclose(fp);
-            sec_free((void*)key1);
+            if (bound != NULL) free_boundtgt(bound);
+            if (fp != NULL) fclose(fp);
+            if (key1 != NULL) {
+                sec_free((void*)key1);
+                key1 = NULL;
+            }
         }
 
         km->free_algs();
-        free((void*)keyinfo.cipheralg);
-        free((void*)keyinfo.digestalg);
+        free_tgtdefn(target);
     }
     CM_ASSERT_DIFFERENT(keylen, 0);
 
     CM_TEST_OK(context);
 }
 
+
+int km_archive_keys(void)
+    /* Generate fixed-pattern keys for testing reading by later releases */
+{   keymanager_t *km;
+    tgtdefn_t *target;
+    bound_tgtdefn_t *bound=NULL;
+    FILE *fp;
+    int flg = 0;
+    char filename[256];
+    km_pw_context_t pw_ctxt;
+
+    pw_ctxt.fd_pw_source = NULL;
+    pw_ctxt.argpasswd[0] = pw_ctxt.argpasswd[1] = km_legacy_passwd;
+
+    build_keymgrs();
+    for (km=keymgrs; km!=NULL; km=km->next) {
+        if ((km->test_flags & CM_READONLY) != 0) continue;
+        km->init_algs();
+
+        target = alloc_tgtdefn(NULL);
+        target->key.format = cm_strdup(km->ident);
+        target->key.filename = NULL;
+        target->key.digestalg = NULL;
+        target->key.cipheralg = NULL;
+        target->key.maxlen = -1;
+
+        bound = alloc_boundtgt(target);
+        km->bind(bound, NULL);
+        sprintf(filename, "/tmp/%s_%s_%s_%s_0", PACKAGE_VERSION, km->ident,
+                        bound->tgt->key.digestalg, bound->tgt->key.cipheralg);
+        bound->tgt->key.filename = cm_strdup(filename);
+
+        /* Write key to file: */
+        fp = fopen(filename, "wb");     /* May need to be "r+b" for LUKS */
+        flg |= km->put_key(bound, &pw_ctxt,
+                        (const unsigned char*)km_legacy_key,
+                        strlen(km_legacy_key), fp);
+        fclose(fp);
+
+        if (bound != NULL) free_boundtgt(bound);
+        km->free_algs();
+        free_tgtdefn(target);
+    }
+
+    return flg;
+}
+
+
+int km_test_legacy(void)
+    /* Check that keyfiles from earlier releases can be read */
+{   DIR *dp;
+    const struct dirent *dent;
+    struct stat sbuff;
+    tgtdefn_t *target;
+    bound_tgtdefn_t *bound=NULL;
+    km_pw_context_t pw_ctxt;
+    unsigned char *key=NULL;
+    int n_legacy = 0, keylen, flg;
+    char *tokbuff, dirpath[1024], keypath[2048];
+
+    CM_TEST_START("Legacy key support");
+
+    pw_ctxt.fd_pw_source = NULL;
+    pw_ctxt.argpasswd[0] = pw_ctxt.argpasswd[1] = km_legacy_passwd;
+
+    snprintf(dirpath, sizeof(dirpath), CM_SRCDIR "/testing/keys");
+    dp = opendir(dirpath);
+    if (dp == NULL) CM_TEST_FAIL();
+
+    for (;;) {
+        dent = readdir(dp);
+        if (dent == NULL) break;
+
+        snprintf(keypath, sizeof(keypath), "%s/%s", dirpath, dent->d_name);
+        if (stat(keypath, &sbuff) != 0) CM_TEST_FAIL();
+        if (!S_ISREG(sbuff.st_mode)) continue;
+        ++n_legacy;
+
+        tokbuff = cm_strdup(dent->d_name);
+        (void)strtok(tokbuff, "_");
+
+        target = alloc_tgtdefn(NULL);
+        target->key.format = cm_strdup(strtok(NULL, "_"));
+        target->key.filename = cm_strdup(keypath);
+        target->key.digestalg = cm_strdup(strtok(NULL, "_"));
+        target->key.cipheralg = cm_strdup(strtok(NULL, "_"));
+
+        bound = bind_tgtdefn(target);
+        if (bound == NULL) CM_TEST_FAIL();
+
+        key = NULL;
+        keylen = 0;
+        flg = cm_get_key(bound, &pw_ctxt, &key, &keylen);
+        if (flg == ERR_BADDEVICE && getuid() != 0) {
+            fprintf(stderr, "Skipping \"%s\" (after %d keys) - possibly requires root privileges\n", keypath, (n_legacy - 1));
+            goto skip_jail;
+        }
+        CM_ASSERT_EQUAL(ERR_NOERROR, flg);
+
+        CM_ASSERT_EQUAL(strlen(km_legacy_key), keylen);
+        if (key != NULL) {
+            CM_ASSERT_EQUAL(0, memcmp(key, km_legacy_key, (size_t)keylen));
+        } else {
+            CM_TEST_FAIL();
+        }
+#if 0
+        fprintf(stderr, "%s -> %s  %d\n", keypath, bound->keymgr->ident, flg);
+#endif
+
+  skip_jail:
+
+        if (key != NULL) sec_free((void*)key);
+        if (bound != NULL) free_boundtgt(bound);
+        if (target != NULL) free_tgtdefn(target);
+
+        free((void*)tokbuff);
+    }
+    closedir(dp);
+
+    if (n_legacy < 4) CM_TEST_FAIL();
+
+    CM_TEST_OK();
+}
+
 #endif  /* TESTING */
 
 
 
 /*
- *  ==== miscellaneous routines ====
+ *  ==== Miscellaneous routines ====
  */
 
 size_t mk_key_string(const unsigned char *key, const size_t keylen, char *buff)
@@ -645,18 +740,18 @@ static int sy_path(const char *path)
     return ERR_INSECURE;
 }
 
-int sycheck_target(const cment_t *cment)
+int sycheck_target(const tgtdefn_t *tgt)
     /* Check that paths within target-specification are sensible */
 {   int eflag=ERR_NOERROR;
 
-    if (cment == NULL) return 0;
+    if (tgt == NULL) return 0;
 
-    eflag |= sy_path(cment->dev);
-    eflag |= sy_path(cment->dir);
-    eflag |= sy_path(cment->key.filename);
+    eflag |= sy_path(tgt->dev);
+    eflag |= sy_path(tgt->dir);
+    eflag |= sy_path(tgt->key.filename);
 
     if (eflag != ERR_NOERROR) {
-        fprintf(stderr, "Specification for target \"%s\" contains non-absolute pathname\n", cment->ident);
+        fprintf(stderr, _("Specification for target \"%s\" contains non-absolute pathname\n"), tgt->ident);
     }
 
     return eflag;
@@ -735,5 +830,5 @@ int cm_mutex_unlock(void)
 /* vim: set ts=4 sw=4 et: */
 
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
