@@ -1,7 +1,7 @@
 /*
- *  declarations for encryption/security mechanisms for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  Declarations for encryption/security mechanisms for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -22,12 +22,14 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
 #ifndef _ARMOUR_H
 #define _ARMOUR_H
 
+#include "cryptmount.h"
+
+
 struct keyinfo;
-struct cment;
+struct bound_tgtdefn;
 struct cm_testinfo;
 struct km_pw_context;
 struct km_overrides;
@@ -40,32 +42,22 @@ typedef struct keymanager {
     int (*init_algs)(void);
     int (*free_algs)(void);
 
-    /* Replace NULL algo-fields in keyinfo with suitable defaults: */
-    void (*mk_default)(struct keyinfo *keyinfo);
-
-    /* Determine whether this keymanager can handle given key: */
-    int (*is_compat)(const struct keyinfo *keyinfo, FILE *fp_key);
+    /*  Attempt to attach to particular target,
+        installing default fields in target-definition */
+    int (*bind)(struct bound_tgtdefn *boundtgt, FILE *fp_key);
 
     /* Get properties, e.g. whether a password is needed for access: */
-    unsigned (*get_properties)(const struct keyinfo *keyinfo);
+    unsigned (*get_properties)(const struct bound_tgtdefn *boundtgt);
 
     /* Extract encrypted key from file: */
-    int (*get_key)(const char *ident, const struct keyinfo *keyinfo,
+    int (*get_key)(struct bound_tgtdefn *boundtgt,
                     const struct km_pw_context *pw_ctxt,
-                    unsigned char **key, int *keylen, FILE *fp_key,
-                    struct km_overrides *overrides);
+                    unsigned char **key, int *keylen, FILE *fp_key);
 
     /* Write encrypted key into file: */
-    int (*put_key)(const char *ident, const struct keyinfo *keyinfo,
+    int (*put_key)(struct bound_tgtdefn *boundtgt,
                     const struct km_pw_context *pw_ctxt,
-                    const unsigned char *key, const int keylen, FILE *fp_key,
-                    const struct km_overrides *overrides);
-
-    /* Message-digest computation (used as source of entropy): */
-    void* (*md_prepare)(void);
-    void (*md_block)(void *state, unsigned char *buff, size_t len);
-    void (*md_final)(void *state, unsigned char **mdval, size_t *mdlen);
-    void (*md_release)(void *state);
+                    const unsigned char *key, const int keylen, FILE *fp_key);
 
     /* Linked-list scaffolding: */
     struct keymanager *next;
@@ -80,57 +72,43 @@ typedef struct keymanager {
 /* Key-manager key-properties flags: */
 enum {
     KM_PROP_HASPASSWD =     0x001,      /* Password needed to access key */
-    KM_PROP_FIXEDLOC =      0x002,      /* Key-file cannot be renamed */
-    KM_PROP_FORMATTED =     0x004       /* Key-file has been formatted */
+    KM_PROP_NEEDSKEYFILE =  0x002,      /* Key-file must be present */
+    KM_PROP_FIXEDLOC =      0x004,      /* Key-file cannot be renamed */
+    KM_PROP_FORMATTED =     0x008       /* Key-file has been formatted */
 };
 
 
-/* Allow key-manager to override some parameters in struct cment: */
-typedef struct km_overrides {
-    unsigned mask;              /* bitwise-or of KM_OR_??? flags */
-    int64_t start, length;
-    char *cipher;
-    int64_t ivoffset;
+/* Association of user-defined target-data & particular key-manager: */
+typedef struct bound_tgtdefn
+{
+    tgtdefn_t *tgt;
+
+    const keymanager_t *keymgr;
     void *km_data;
-} km_overrides_t;
-
-enum {
-    KM_OR_START =       0x001,
-    KM_OR_CIPHER =      0x002,
-    KM_OR_IVOFFSET =    0x004
-};
+} bound_tgtdefn_t;
 
 
 const char **get_keymgr_list(void);
-keymanager_t *get_keymanager(const struct keyinfo *keyinfo, FILE *fp_key);
-
-
-keymanager_t *kmblti_gethandle(void);
-keymanager_t *kmgcry_gethandle(void);
-keymanager_t *kmluks_gethandle(void);
-keymanager_t *init_keymanager(keymanager_t *km);
 int free_keymanagers(void);
 
-unsigned cm_get_keyproperties(const struct keyinfo *keyinfo);
-int cm_get_key(const char *ident, const struct keyinfo *keyinfo,
+
+bound_tgtdefn_t *bind_tgtdefn(const tgtdefn_t *tgt);
+void free_boundtgt(bound_tgtdefn_t *boundtgt);
+
+
+unsigned cm_get_keyproperties(const bound_tgtdefn_t *boundtgt);
+int cm_get_key(bound_tgtdefn_t *boundtgt,
             const struct km_pw_context *pw_ctxt,
-            unsigned char **key, int *keylen,
-            km_overrides_t *overrides);
-int cm_put_key(const char *ident, const struct keyinfo *keyinfo,
+            unsigned char **key, int *keylen);
+int cm_put_key(bound_tgtdefn_t *boundtgt,
             const struct km_pw_context *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key,
-            const km_overrides_t *overrides);
+            const unsigned char *key, const int keylen, FILE *fp_key);
 size_t mk_key_string(const unsigned char *key, const size_t keylen,
             char *buff);
 
-km_overrides_t *alloc_overrides();
-void merge_overrides(const struct cment *ent,
-            const km_overrides_t *overrides, struct cment *merged);
-void free_overrides(km_overrides_t *overrides);
-
 int sycheck_directory(const char *dirname);
 int sycheck_cmtab(const char *cmtab);
-int sycheck_target(const struct cment *ent);
+int sycheck_target(const struct tgtdefn *ent);
 
 int cm_mutex_lock(void);
 int cm_mutex_unlock(void);
@@ -154,5 +132,5 @@ int cm_mutex_unlock(void);
 /* vim: set ts=4 sw=4 et: */
 
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
