@@ -2,7 +2,7 @@
  *  Methods for "Blowfish" cipher algorithm
  *  (Based on Bruce Schneier's implementation at http://www.schneier.com,
  *  subsequently edited by RW Penney for "cryptmount")
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
  */
 
 /*
@@ -379,7 +379,6 @@ cm_bf_ctxt_t *cm_bf_init(unsigned char *key, size_t keybytes)
     memcpy((void*)ctxt->p, (const void*)bf_p, sizeof(bf_p));
     memcpy((void*)ctxt->sbox, (const void*)bf_sbox, sizeof(bf_sbox));
 
-
     j = 0;
     for (i=0; i<CM_BF_nrounds+2; ++i) {
         data = 0;
@@ -445,7 +444,7 @@ int bf_test_blowfish()
     uint32_t vals[2];
     unsigned idx;
 
-    CM_TEST_START("internal Blowfish");
+    CM_TEST_START("Internal Blowfish");
 
     idx = 0;
     while (tvecs[idx].key != NULL) {
@@ -472,3 +471,5 @@ int bf_test_blowfish()
 }
 
 #endif  /* TESTING */
+
+/* vim: set ts=4 sw=4 et: */
