@@ -1,7 +1,7 @@
 /*
- *  declarations for unit-test utilities for cryptmoumt
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2006-2008, RW Penney
+ *  Declarations for unit-test utilities for cryptmoumt
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2006-2009, RW Penney
  */
 
 /*
@@ -92,7 +92,7 @@ extern cm_testinfo_t *test_ctxtptr;
 
 #define CM_TEST_FAIL(TI) \
     { \
-        fprintf(stderr, " FAILED\n"); \
+        fprintf(stderr, " FAILED at %s:%d\n", __FILE__, __LINE__); \
         if (test_ctxtptr->current != NULL) { \
             fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
         ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
@@ -101,7 +101,7 @@ extern cm_testinfo_t *test_ctxtptr;
 
 #define CM_TEST_ABORT(TI) \
     { \
-        fprintf(stderr, " ABORTED\n"); \
+        fprintf(stderr, " ABORTED at %s:%d\n", __FILE__, __LINE__); \
         if (test_ctxtptr->current != NULL) { \
             fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
         ++test_ctxtptr->test_stats[CM_TEST_ABORTED]; \
@@ -114,6 +114,8 @@ int cm_run_tests();
 
 #endif  /* _CMTEST_H */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  (C)Copyright 2006-2008, RW Penney
+ *  (C)Copyright 2006-2009, RW Penney
  */
