@@ -1,7 +1,7 @@
 /*
- *  general declarations for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  General declarations for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -57,7 +57,7 @@
 #endif
 
 
-enum    /* exit-codes */
+enum    /* Exit-codes */
 {
     EXIT_OK =       0,
     EXIT_BADOPT =   1,
@@ -68,7 +68,7 @@ enum    /* exit-codes */
 };
 
 
-enum    /* error flags */
+enum    /* Error flags */
 {
     ERR_NOERROR =       0,
 
@@ -76,12 +76,13 @@ enum    /* error flags */
     WRN_NOPASSWD,
     WRN_LOWENTROPY,
 
-    ERR_threshold = 0x10,       /* dividing-line between warnings & errors */
+    ERR_threshold = 0x10,       /* Dividing-line between warnings & errors */
 
+    ERR_NOTSUPPORTED,
     ERR_BADKEYFORMAT,
     ERR_BADALGORITHM,
-    ERR_BADFILE,                /* serious problem with accessing file */
-    ERR_BADDECRYPT,             /* failure to extract cipher key from file */
+    ERR_BADFILE,                /* Serious problem with accessing file */
+    ERR_BADDECRYPT,             /* Failure to extract cipher key from file */
     ERR_BADENCRYPT,
     ERR_MEMSPACE,
     ERR_DMSETUP,
@@ -100,7 +101,7 @@ enum    /* error flags */
 };
 
 
-enum    /* target configuration switches */
+enum    /* Target configuration switches */
 {
     FLG_USER =      0x001,
     FLG_FSCK =      0x002,
@@ -112,37 +113,39 @@ enum    /* target configuration switches */
 
 typedef struct keyinfo
 {
-    char *format;           /* type of key file, e.g. 'raw', 'libgcrypt' */
+    const char *format;     /* Type of key file, e.g. 'raw', 'libgcrypt' */
     char *filename;
     char *digestalg;
     char *cipheralg;
-    long maxlen;            /* maximum number of bytes to read from keyfile */
-    unsigned retries;       /* limit on password-entry attempts */
+    long maxlen;            /* Maximum number of bytes to read from keyfile */
+    unsigned retries;       /* Limit on password-entry attempts */
 } keyinfo_t;
 
 
-typedef struct cment        /* entry in list of cryptmount filesystem-targets */
+typedef struct tgtdefn      /* Entry in list of cryptmount filesystem-targets */
 {
-    char *ident;            /* unique identifying name */
-    unsigned flags;         /* configuration switches */
+    const char *ident;      /* Unique identifying name */
+    unsigned flags;         /* Configuration switches */
 
-    char *dev;              /* device node or raw file */
-    int64_t start, length;  /* starting sector + num of sectors (or 0, -1) */
-    char *dir;              /* mount-point */
-    char *fstype;           /* filesystem type */
-    char *fsoptions;        /* mount options */
-    char *loopdev;          /* loopback device to wrap around raw file */
+    char *dev;              /* Device node or raw file */
+    int64_t start, length;  /* Starting sector + num of sectors (or 0, -1) */
+    char *dir;              /* Mount-point */
+    char *fstype;           /* Filesystem type */
+    char *fsoptions;        /* Mount options */
+    char *loopdev;          /* Loopback device to wrap around raw file */
 
-    char *cipher;           /* cipher used on filesystem */
-    int64_t ivoffset;       /* cipher initialization-vector offset */
+    char *cipher;           /* Cipher used on filesystem */
+    int64_t ivoffset;       /* Cipher initialization-vector offset */
 
-    keyinfo_t key;          /* location/format of key */
+    keyinfo_t key;          /* Location/format of key */
 
-    struct cment *nx;       /* form into linked list */
-} cment_t;
+    struct tgtdefn *nx;       /* Form into linked list */
+} tgtdefn_t;
 
 #endif  /* _CRYPTMOUNT_H */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
