@@ -1,7 +1,7 @@
 /*
- *  declarations for filesytem-related utilities for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  Declarations for filesytem-related utilities for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -26,20 +26,19 @@
 #ifndef _FSUTILS_H
 #define _FSUTILS_H
 
-struct cment;
+struct tgtdefn;
 
 int parse_fsoptions(const char *buff, unsigned long *mflags);
-int fs_check(const char *dev, const struct cment *ent);
-int fs_mount(const char *dev, const struct cment *ent);
-int fs_unmount(const struct cment *ent);
-int fs_swapon(const char *dev, const struct cment *ent);
-int fs_swapoff(const struct cment *ent);
-int is_mounted(const struct cment *ent);
+int fs_check(const char *dev, const struct tgtdefn *tgt);
+int fs_mount(const char *dev, const struct tgtdefn *tgt);
+int fs_unmount(const struct tgtdefn *tgt);
+int fs_swapon(const char *dev, const struct tgtdefn *tgt);
+int fs_swapoff(const struct tgtdefn *tgt);
+int is_mounted(const struct tgtdefn *tgt);
 int is_readonlyfs(const char *path);
 
 #endif  /* _FSUTILS_H */
 
 /*
- *  fsutils.h
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
