@@ -1,7 +1,7 @@
 /*
  *  loopback-device utilities for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -196,7 +196,7 @@ int loop_dellist(unsigned devcnt, const dev_t *devids)
     for (i=0; i<devcnt; ++i) {
         if (loop_ident(major(devids[i]), minor(devids[i]), buff, sizeof(buff))
           || (loop_destroy(buff) != ERR_NOERROR)) {
-            fprintf(stderr, _("failed to free device (%d,%d)\n"),
+            fprintf(stderr, _("Failed to free device (%d,%d)\n"),
                         major(devids[i]), minor(devids[i]));
             eflag = 1;
         }
@@ -232,7 +232,7 @@ int blockify_file(const char *filename, int fmode, const char *prefdev,
         } else {
             loopdev = (char*)malloc((size_t)BUFFMIN);
             if (loop_findfree(loopdev, (size_t)BUFFMIN) != 0) {
-                fprintf(stderr, _("no available loopback devices\n"));
+                fprintf(stderr, _("No available loopback devices\n"));
                 eflag = ERR_BADDEVICE;
                 goto bail_out;
             }
@@ -246,7 +246,7 @@ int blockify_file(const char *filename, int fmode, const char *prefdev,
         *isloop = 1;
     } else {
         fprintf(stderr,
-            _("bad device type (%x) for \"%s\" (need block/file)\n"),
+            _("Bad device type (%x) for \"%s\" (need block/file)\n"),
             (unsigned)sbuff.st_mode, filename);
         *devname = NULL;
         *isloop = 0;
@@ -275,5 +275,5 @@ int unblockify_file(const char **devname, int isloop)
 }
 
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
