@@ -1,7 +1,7 @@
 /*
- *  config-table and mount-table utilities for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  Config-table and mount-table utilities for cryptmount
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -57,7 +57,7 @@ struct statfile     /* information about status file */
 };
 
 
-static cment_t *parse_stream(FILE *fp, const char *cfgname);
+static tgtdefn_t *parse_stream(FILE *fp, const char *cfgname);
 
 
 /*  The format for the status-file (version 0) is as follows:
@@ -68,47 +68,47 @@ static cment_t *parse_stream(FILE *fp, const char *cfgname);
 */
 
 
-cment_t *alloc_cment(const cment_t *prototype)
-    /* allocate storage for a target-structure */
-{   cment_t *cment;
+tgtdefn_t *alloc_tgtdefn(const tgtdefn_t *prototype)
+    /* Allocate storage for a target-structure */
+{   tgtdefn_t *tgt;
 
 #define DFLT_OR_PROTO(dflt, proto) \
         (prototype == NULL ? (dflt) : (proto))
 
-    cment = (cment_t*)malloc(sizeof(cment_t));
+    tgt = (tgtdefn_t*)malloc(sizeof(tgtdefn_t));
 
-    cment->ident = NULL;
-    cment->flags = DFLT_OR_PROTO(FLG_DEFAULTS, prototype->flags);
+    tgt->ident = NULL;
+    tgt->flags = DFLT_OR_PROTO(FLG_DEFAULTS, prototype->flags);
 
-    cment->dev = NULL;
-    cment->start = 0; cment->length = -1;
-    cment->dir = NULL;
-    cment->fstype = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fstype));
-    cment->fsoptions = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fsoptions));
-    cment->loopdev = NULL;
+    tgt->dev = NULL;
+    tgt->start = 0; tgt->length = -1;
+    tgt->dir = NULL;
+    tgt->fstype = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fstype));
+    tgt->fsoptions = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fsoptions));
+    tgt->loopdev = NULL;
 
-    cment->cipher = DFLT_OR_PROTO(cm_strdup(DFLT_CIPHER),
+    tgt->cipher = DFLT_OR_PROTO(cm_strdup(DFLT_CIPHER),
                                 cm_strdup(prototype->cipher));
-    cment->ivoffset = 0;
+    tgt->ivoffset = 0;
 
-    cment->key.format = DFLT_OR_PROTO(NULL, cm_strdup(prototype->key.format));
-    cment->key.filename = NULL;
-    cment->key.digestalg = DFLT_OR_PROTO(NULL, cm_strdup(prototype->key.digestalg));
-    cment->key.cipheralg = DFLT_OR_PROTO(NULL, cm_strdup(prototype->key.cipheralg));
-    cment->key.maxlen = DFLT_OR_PROTO(-1L, prototype->key.maxlen);
-    cment->key.retries = DFLT_OR_PROTO(1U, prototype->key.retries);
+    tgt->key.format = DFLT_OR_PROTO(NULL, cm_strdup(prototype->key.format));
+    tgt->key.filename = NULL;
+    tgt->key.digestalg = DFLT_OR_PROTO(NULL, cm_strdup(prototype->key.digestalg));
+    tgt->key.cipheralg = DFLT_OR_PROTO(NULL, cm_strdup(prototype->key.cipheralg));
+    tgt->key.maxlen = DFLT_OR_PROTO(-1L, prototype->key.maxlen);
+    tgt->key.retries = DFLT_OR_PROTO(1U, prototype->key.retries);
 
-    cment->nx = NULL;
+    tgt->nx = NULL;
 
 #undef DFLT_OR_PROTO
 
-    return cment;
+    return tgt;
 }
 
 
-const cment_t *get_cment(const cment_t *head, const char *ident)
-    /* find info-structure for target of given name */
-{   const cment_t *itr,*ent=NULL;
+const tgtdefn_t *get_tgtdefn(const tgtdefn_t *head, const char *ident)
+    /* Find info-structure for target of given name */
+{   const tgtdefn_t *itr,*ent=NULL;
 
     for (itr=head; itr!=NULL && ent==NULL; itr=itr->nx) {
         if (strcmp(ident, itr->ident) == 0) {
@@ -120,30 +120,65 @@ const cment_t *get_cment(const cment_t *head, const char *ident)
 }
 
 
-void free_cment(cment_t *cment)
-    /* relinquish storage for a target-structure */
+tgtdefn_t *clone_tgtdefn(const tgtdefn_t *orig)
+    /* Create (deep) copy of target-definition */
+{   tgtdefn_t *clone;
+
+    if (orig == NULL) return NULL;
+
+    clone = (tgtdefn_t*)malloc(sizeof(tgtdefn_t));
+
+    clone->ident = cm_strdup(orig->ident);
+    clone->flags = orig->flags;
+
+    clone->dev = cm_strdup(orig->dev);
+    clone->start = orig->start;
+    clone->length = orig->length;
+    clone->dir = cm_strdup(orig->dir);
+    clone->fstype = cm_strdup(orig->fstype);
+    clone->fsoptions = cm_strdup(orig->fsoptions);
+    clone->loopdev = cm_strdup(orig->loopdev);
+
+    clone->cipher = cm_strdup(orig->cipher);
+    clone->ivoffset = orig->ivoffset;
+
+    clone->key.format = cm_strdup(orig->key.format);
+    clone->key.filename = cm_strdup(orig->key.filename);
+    clone->key.digestalg = cm_strdup(orig->key.digestalg);
+    clone->key.cipheralg = cm_strdup(orig->key.cipheralg);
+    clone->key.maxlen = orig->key.maxlen;
+    clone->key.retries = orig->key.retries;
+
+    clone->nx = NULL;
+
+    return clone;
+}
+
+
+void free_tgtdefn(tgtdefn_t *tgt)
+    /* Relinquish storage for a target-structure */
 {
-    if (cment->ident != NULL) free((void*)cment->ident);
+    if (tgt->ident != NULL) free((void*)tgt->ident);
 
-    if (cment->dev != NULL) free((void*)cment->dev);
-    if (cment->dir != NULL) free((void*)cment->dir);
-    if (cment->fstype != NULL) free((void*)cment->fstype);
-    if (cment->fsoptions != NULL) free((void*)cment->fsoptions);
-    if (cment->loopdev != NULL) free((void*)cment->loopdev);
+    if (tgt->dev != NULL) free((void*)tgt->dev);
+    if (tgt->dir != NULL) free((void*)tgt->dir);
+    if (tgt->fstype != NULL) free((void*)tgt->fstype);
+    if (tgt->fsoptions != NULL) free((void*)tgt->fsoptions);
+    if (tgt->loopdev != NULL) free((void*)tgt->loopdev);
 
-    if (cment->cipher != NULL) free((void*)cment->cipher);
+    if (tgt->cipher != NULL) free((void*)tgt->cipher);
 
-    if (cment->key.format != NULL) free((void*)cment->key.format);
-    if (cment->key.filename != NULL) free((void*)cment->key.filename);
-    if (cment->key.digestalg != NULL) free((void*)cment->key.digestalg);
-    if (cment->key.cipheralg != NULL) free((void*)cment->key.cipheralg);
+    if (tgt->key.format != NULL) free((void*)tgt->key.format);
+    if (tgt->key.filename != NULL) free((void*)tgt->key.filename);
+    if (tgt->key.digestalg != NULL) free((void*)tgt->key.digestalg);
+    if (tgt->key.cipheralg != NULL) free((void*)tgt->key.cipheralg);
 
-    free((void*)cment);
+    free((void*)tgt);
 }
 
 
 static void append(char c, char **buff, unsigned *pos, unsigned *bufflen)
-    /* append character to string, reallocating memory as necessary */
+    /* Append character to string, reallocating memory as necessary */
 {   unsigned newlen;
 
     if (*pos >= *bufflen) {
@@ -158,7 +193,7 @@ static void append(char c, char **buff, unsigned *pos, unsigned *bufflen)
 
 
 static int proc_string(void *ptr, const char *src)
-    /* process var=val entry in config-table for "string" type */
+    /* Process var=val entry in config-table for "string" type */
 {   char **addr=(char**)ptr;
 
     *addr = (char*)realloc(*addr, (size_t)(strlen(src) + 1));
@@ -170,7 +205,7 @@ static int proc_string(void *ptr, const char *src)
 
 #define PROC_VARVAL(FN_NAME, TYPE, FMT)                         \
     static int FN_NAME(void *ptr, const char *src)              \
-        /* process var=val entry in config-table for "TYPE" */  \
+        /* Process var=val entry in config-table for "TYPE" */  \
     {   TYPE qv, *addr=(TYPE*)ptr;                              \
         if (sscanf(src, FMT, &qv) == 1) {                       \
             *addr = qv; return 0;                               \
@@ -186,7 +221,7 @@ PROC_VARVAL(proc_int64, int64_t, "%" SCNi64)
 
 
 int proc_flags(void *ptr, const char *src)
-    /* convert string of configuration-switches into binary flags */
+    /* Convert string of configuration-switches into binary flags */
 {   unsigned *flags=(unsigned*)ptr;
     struct opt_t {
         const char *str;
@@ -226,14 +261,14 @@ int proc_flags(void *ptr, const char *src)
 
 
 
-static void read_token(char *buff, unsigned *t_state, cment_t *cment)
-    /* process token (word) from configuration-file while parsing */
+static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
+    /* Process token (word) from configuration-file while parsing */
 {   struct tokinfo_t {
         const char *name;
         int varoffset;
         int (*proc)(void *var, const char *val);
     } *tok;
-#define OFFSET(x) (int)((char*)&((cment_t*)NULL)->x - (char*)NULL)
+#define OFFSET(x) (int)((char*)&((tgtdefn_t*)NULL)->x - (char*)NULL)
     struct tokinfo_t toktable[] = {
         { "flags",          OFFSET(flags),          proc_flags },
         { "dev",            OFFSET(dev),            proc_string },
@@ -258,7 +293,7 @@ static void read_token(char *buff, unsigned *t_state, cment_t *cment)
 
     switch (*t_state) {
         case T_IDENT:
-            (void)proc_string((void*)&cment->ident, buff);
+            (void)proc_string((void*)&tgt->ident, buff);
             *t_state = T_LBRACE;
             break;
         case T_LBRACE:
@@ -266,11 +301,11 @@ static void read_token(char *buff, unsigned *t_state, cment_t *cment)
             break;
         case T_OPT:
             if (strcmp(buff, "}") == 0) {
-                if (cment->key.format == NULL
-                  && cment->key.cipheralg != NULL
-                  && strcmp(cment->key.cipheralg, "none") == 0) {
+                if (tgt->key.format == NULL
+                  && tgt->key.cipheralg != NULL
+                  && strcmp(tgt->key.cipheralg, "none") == 0) {
                     fprintf(stderr, _("cryptmount: using \"keycipher=none\" is deprecated - please use \"keyformat=raw\" instead\n"));
-                    cment->key.format = cm_strdup("raw");
+                    tgt->key.format = cm_strdup("raw");
                 }
                 *t_state = T_RBRACE; break;
             }
@@ -279,15 +314,15 @@ static void read_token(char *buff, unsigned *t_state, cment_t *cment)
             }
             *eq = '\0'; ++eq;
 
-            /* delegate processing to specific token-processor from table: */
+            /* Delegate processing to specific token-processor from table: */
             for (tok=toktable; tok->name!=NULL; ++tok) {
                 if (strcmp(tok->name, buff) == 0) {
-                    (*tok->proc)((void*)((char*)cment + tok->varoffset), eq);
+                    (*tok->proc)((void*)((char*)tgt + tok->varoffset), eq);
                     break;
                 }
             }
             if (tok->name == NULL) {
-                fprintf(stderr, _("unrecognized option \"%s\"\n"), buff);
+                fprintf(stderr, _("Unrecognized option \"%s\"\n"), buff);
                 *t_state = T_ERROR;
             }
             break;
@@ -297,31 +332,31 @@ static void read_token(char *buff, unsigned *t_state, cment_t *cment)
 }
 
 
-cment_t *parse_stream(FILE *fp, const char *cfgname)
-    /* convert config-file information into linked-list of target-structures */
+tgtdefn_t *parse_stream(FILE *fp, const char *cfgname)
+    /* Convert config-file information into linked-list of target-structures */
 {   enum { C_SPACE, C_WORD, C_COMMENT };
     int ch,lineno=1,literal=0;
     unsigned pos=0,bufflen=0;
     unsigned c_state,t_state;
     char *buff=NULL;
-    cment_t *head=NULL, **cmp=&head, *cment=NULL, *prototype=NULL;
+    tgtdefn_t *head=NULL, **cmp=&head, *tgt=NULL, *prototype=NULL;
 
     c_state = C_SPACE;
     t_state = T_IDENT;
-    cment = alloc_cment(prototype);
+    tgt = alloc_tgtdefn(prototype);
 
     while (!feof(fp) && t_state != T_ERROR) {
         ch = fgetc(fp);
         if (ch == (int)'\n') ++lineno;
 
-        if (literal && ch == (int)'\n') {    /* ignore escaped end-of-line */
+        if (literal && ch == (int)'\n') {    /* Ignore escaped end-of-line */
             literal = 0;
             continue;
         }
 
         if (!literal && ch == (int)'#') c_state = C_COMMENT;
         if (!literal && ch == (int)'\\') {
-            /* treat next character literally */
+            /* Treat next character literally */
             literal = 1;
             continue;
         }
@@ -339,7 +374,7 @@ cment_t *parse_stream(FILE *fp, const char *cfgname)
                     append((char)ch, &buff, &pos, &bufflen);
                 } else {
                     append('\0', &buff, &pos, &bufflen);
-                    read_token(buff, &t_state, cment);
+                    read_token(buff, &t_state, tgt);
                     c_state = C_SPACE;
                 }
                 break;
@@ -352,40 +387,40 @@ cment_t *parse_stream(FILE *fp, const char *cfgname)
 
         literal = 0;
         if (t_state == T_RBRACE) {
-            /* parsing has reached end of target definition */
+            /* Parsing has reached end of target definition */
 
-            if (strcmp(cment->ident, "_DEFAULTS_") == 0) {
-                /* new target definition is set of default values */
-                if (prototype != NULL) free_cment(prototype);
-                prototype = cment;
+            if (strcmp(tgt->ident, "_DEFAULTS_") == 0) {
+                /* New target definition is set of default values */
+                if (prototype != NULL) free_tgtdefn(prototype);
+                prototype = tgt;
             } else {
-                /* new target definition is genuine target */
-                *cmp = cment;
-                cmp = &cment->nx;
+                /* New target definition is genuine target */
+                *cmp = tgt;
+                cmp = &tgt->nx;
             }
 
-            cment = alloc_cment(prototype);
+            tgt = alloc_tgtdefn(prototype);
             t_state = T_IDENT;
         }
     }
 
     if (t_state == T_ERROR) {
-        fprintf(stderr, _("configuration error near %s:%d\n"),
+        fprintf(stderr, _("Configuration error near %s:%d\n"),
                 cfgname, lineno);
     }
 
-    if (prototype != NULL) free_cment(prototype);
-    if (cment != NULL) free_cment(cment);
+    if (prototype != NULL) free_tgtdefn(prototype);
+    if (tgt != NULL) free_tgtdefn(tgt);
     if (buff != NULL) free((void*)buff);
 
     return head;
 }
 
 
-cment_t *parse_config(const char *cfgname)
-    /* convert config-file into linked-list of target-structures */
+tgtdefn_t *parse_config(const char *cfgname)
+    /* Convert config-file into linked-list of target-structures */
 {   FILE *fp;
-    cment_t *head=NULL;
+    tgtdefn_t *head=NULL;
 
     fp = fopen(cfgname, "r");
     if (fp == NULL) {
@@ -400,10 +435,10 @@ cment_t *parse_config(const char *cfgname)
 }
 
 
-cment_t *parse_config_fd(int fd)
-    /* convert input-stream config-data into target-structures */
+tgtdefn_t *parse_config_fd(int fd)
+    /* Convert input-stream config-data into target-structures */
 {   FILE *fp=NULL;
-    cment_t *head=NULL;
+    tgtdefn_t *head=NULL;
     char label[64];
 
     fp = fdopen(fd, "r");
@@ -419,22 +454,22 @@ cment_t *parse_config_fd(int fd)
 }
 
 
-void free_config(cment_t **head)
-    /* free all entries in target-config list */
-{   cment_t *cmx;
+void free_config(tgtdefn_t **head)
+    /* Free all entries in target-config list */
+{   tgtdefn_t *cmx;
 
     if (head == NULL) return;
 
     while ((cmx = *head) != NULL) {
         *head = cmx->nx;
-        free_cment(cmx);
+        free_tgtdefn(cmx);
     }
 }
 
 
 
-tgtstat_t *alloc_tgtstatus(const struct cment *ent)
-    /* create new status record for given target */
+tgtstat_t *alloc_tgtstatus(const tgtdefn_t *tgt)
+    /* Create new status record for given target */
 {   tgtstat_t *ts;
 
     ts = (tgtstat_t*)malloc(sizeof(tgtstat_t));
@@ -442,17 +477,14 @@ tgtstat_t *alloc_tgtstatus(const struct cment *ent)
     ts->uid = 0;
     ts->nx = NULL;
 
-    if (ent != NULL) {
-        ts->ident = (char*)malloc((size_t)(strlen(ent->ident) + 1));
-        strcpy(ts->ident, ent->ident);
-    }
+    if (tgt != NULL) ts->ident = cm_strdup(tgt->ident);
 
     return ts;
 }
 
 
 void free_tgtstatus(tgtstat_t *ts)
-    /* free storage of target-status record (or list thereof) */
+    /* Free storage of target-status record (or list thereof) */
 {   tgtstat_t *tx=NULL;
 
     while ((tx = ts) != NULL) {
@@ -464,7 +496,7 @@ void free_tgtstatus(tgtstat_t *ts)
 
 
 struct statfile *statfile_open(const char *fname, const char *mode)
-    /* prepare status-file for reading/writing */
+    /* Prepare status-file for reading/writing */
 {   struct statfile *sf=NULL;
     char buff[256];
     FILE *fp;
@@ -474,12 +506,15 @@ struct statfile *statfile_open(const char *fname, const char *mode)
         sf->fp = fp;
 
         if (mode[0] == 'w') {
-            sf->version = 0;        /* format-version for new files */
+            sf->version = 0;        /* Format-version for new files */
             fprintf(fp,"# auto-generated by cryptmount - do not edit\n");
             fprintf(fp, "%d\n", sf->version);
         } else {
-            (void)fgets(buff, (int)sizeof(buff), fp);
-            fscanf(fp, "%d", &sf->version);
+            if (fgets(buff, (int)sizeof(buff), fp) == NULL
+              || fscanf(fp, "%d", &sf->version) != 1) {
+                free((void*)sf);
+                sf = NULL;
+            }
         }
     }
 
@@ -488,7 +523,7 @@ struct statfile *statfile_open(const char *fname, const char *mode)
 
 
 tgtstat_t *statfile_read(struct statfile *sf)
-    /* read information about next target from status-file */
+    /* Read information about next target from status-file */
 {   tgtstat_t *ts=NULL;
     char *ident=NULL;
     int len;
@@ -499,7 +534,7 @@ tgtstat_t *statfile_read(struct statfile *sf)
     if (fscanf(sf->fp, "%d,", &len) != 1) goto bail_out;
 
     ident = (char*)malloc((size_t)(len + 1));
-    if (fread((void*)ident, (size_t)(len + 1), (size_t)1, sf->fp) != 1) {
+    if (cm_fread((void*)ident, (size_t)(len + 1), sf->fp) != 0) {
         goto bail_out;
     }
     ident[len] = '\0';
@@ -535,8 +570,8 @@ void statfile_close(struct statfile *sf)
 }
 
 
-tgtstat_t *get_tgtstatus(const cment_t *cment)
-    /* find mount/owner status of given target */
+tgtstat_t *get_tgtstatus(const tgtdefn_t *tgt)
+    /* Find mount/owner status of given target */
 {   char *fname=NULL;
     tgtstat_t *ts=NULL;
     struct statfile *sf;
@@ -548,7 +583,7 @@ tgtstat_t *get_tgtstatus(const cment_t *cment)
     if (sf == NULL) goto bail_out;
 
     while ((ts = statfile_read(sf)) != NULL) {
-        if (strcmp(cment->ident, ts->ident) == 0) break;
+        if (strcmp(tgt->ident, ts->ident) == 0) break;
         else free_tgtstatus(ts);
     }
 
@@ -564,7 +599,7 @@ tgtstat_t *get_tgtstatus(const cment_t *cment)
 
 
 tgtstat_t *get_all_tgtstatus()
-    /* find list of mount/owner status for all mounted targets */
+    /* Find list of mount/owner status for all mounted targets */
 {   char *fname=NULL;
     tgtstat_t *ts=NULL, *head=NULL, **sfp=&head;
     struct statfile *sf;
@@ -591,7 +626,7 @@ tgtstat_t *get_all_tgtstatus()
 }
 
 
-int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
+int put_tgtstatus(const tgtdefn_t *tgt, const tgtstat_t *newstat)
 {   char *newfname=NULL,*oldfname=NULL;
     struct statfile *sfin,*sfout;
     tgtstat_t *ts=NULL;
@@ -615,9 +650,9 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
             eflag = 1; goto bail_out;
         }
 
-        /* copy most entries from existing status-file: */
+        /* Copy most entries from existing status-file: */
         while ((ts = statfile_read(sfin)) != NULL) {
-            if (strcmp(cment->ident, ts->ident) != 0) {
+            if (strcmp(tgt->ident, ts->ident) != 0) {
                 statfile_write(sfout, ts);
             }
             free_tgtstatus(ts);
@@ -625,16 +660,18 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
         statfile_close(sfin);
     }
 
-    /* add new entry onto end of new status-file: */
+    /* Add new entry onto end of new status-file: */
     if (newstat != NULL) {
         statfile_write(sfout, newstat);
     }
     statfile_close(sfout);
 
-    /* overwrite old status-file: */
-    rename(newfname, oldfname);
-    chown(oldfname, (uid_t)0, (gid_t)0);
-    chmod(oldfname, S_IWUSR|S_IRUSR | S_IRGRP | S_IROTH);
+    /* Overwrite old status-file: */
+    if (rename(newfname, oldfname) != 0
+      || chown(oldfname, (uid_t)0, (gid_t)0) != 0
+      || chmod(oldfname, S_IWUSR|S_IRUSR | S_IRGRP | S_IROTH) != 0) {
+        eflag = 1; goto bail_out;
+    }
 
   bail_out:
 
@@ -645,8 +682,8 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
     return eflag;
 }
 
+/* vim: set ts=4 sw=4 et: */
 
 /*
- *  tables.c
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
