@@ -1,7 +1,7 @@
 /*
  *  declarations for config-table & mount-table utilities for cryptmount
- *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  $Revision: 258 $, $Date: 2009-05-04 07:07:04 +0100 (Mon, 04 May 2009) $
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -27,7 +27,7 @@
 #define _TABLES_H
 
 
-struct cment;
+struct tgtdefn;
 
 typedef struct tgtstat
 {
@@ -38,23 +38,26 @@ typedef struct tgtstat
 } tgtstat_t;
 
 
-struct cment *alloc_cment(const struct cment *prototype);
-const struct cment *get_cment(const struct cment *head, const char *ident);
-void free_cment(struct cment *ent);
+struct tgtdefn *alloc_tgtdefn(const struct tgtdefn *prototype);
+const struct tgtdefn *get_tgtdefn(const struct tgtdefn *head,
+                                const char *ident);
+struct tgtdefn *clone_tgtdefn(const struct tgtdefn *orig);
+void free_tgtdefn(struct tgtdefn *tgt);
 
-struct cment *parse_config(const char *cfgname);
-struct cment *parse_config_fd(int fd);
-void free_config(struct cment **head);
+struct tgtdefn *parse_config(const char *cfgname);
+struct tgtdefn *parse_config_fd(int fd);
+void free_config(struct tgtdefn **head);
 
-tgtstat_t *alloc_tgtstatus(const struct cment *ent);
-tgtstat_t *get_tgtstatus(const struct cment *ent);
+tgtstat_t *alloc_tgtstatus(const struct tgtdefn *tgt);
+tgtstat_t *get_tgtstatus(const struct tgtdefn *tgt);
 tgtstat_t *get_all_tgtstatus();
-int put_tgtstatus(const struct cment *ent, const tgtstat_t *tstat);
+int put_tgtstatus(const struct tgtdefn *tgt, const tgtstat_t *tstat);
 void free_tgtstatus(tgtstat_t *tstat);
 
 #endif  /* _TABLES_H */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  tables.h
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
