@@ -1,7 +1,7 @@
 /*
  *  miscellaneous utilitie functions for cryptmount
  *  $Revision: 134 $, $Date: 2007-02-19 17:18:07 +0000 (Mon, 19 Feb 2007) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -44,7 +44,6 @@
 #endif
 
 
-
 int cm_path(char **buff, const char *file)
     /* Create full pathname of config file */
 {   size_t pfxlen, sfxlen;
@@ -69,17 +68,19 @@ int cm_path(char **buff, const char *file)
 
 
 char *cm_strdup(const char *orig)
-    /* Make duplicate of existing string, reserving memory */
+    /* Make duplicate of existing string, allocating memory for copy */
 {   char *cpy;
 
     if (orig == NULL) return NULL;
+
     cpy = (char*)malloc((size_t)(strlen(orig) + 1));
+
     return strcpy(cpy, orig);
 }
 
 
 int cm_strcasecmp(const char *s1, const char *s2)
-    /* find legigraphical order of s1 & s2, ignoring case */
+    /* Find legigraphical order of s1 & s2, ignoring case */
 {
     if (s1 == NULL || s2 == NULL) return (s1 != NULL) - (s2 != NULL);
     while (*s1 != '\0' && *s2 != '\0' && tolower(*s1) == tolower(*s2)) {
@@ -89,6 +90,49 @@ int cm_strcasecmp(const char *s1, const char *s2)
     return (tolower(*s1) - tolower(*s2));
 }
 
+
+int cm_startswith(const char **str, const char *prefix)
+    /* Check whether *prefix appears at start of **str */
+{   int valid=1;
+
+    if (str == NULL) return 0;
+    if (*str == NULL || prefix == NULL) return (*str == NULL && prefix == NULL);
+
+    while (valid && *prefix != '\0') {
+        valid &= (*prefix == **str);
+        ++prefix;
+        ++*str;
+    }
+
+    return valid;
+}
+
+
+#ifdef TESTING
+
+int ut_test_strops()
+    /* Check basic string operations */
+{   const char *refstr="alphabet", *refp;
+
+    CM_TEST_START("String operations");
+
+    CM_ASSERT_EQUAL(cm_strcasecmp("alpha", "alpha"), 0);
+    CM_ASSERT_EQUAL(cm_strcasecmp("alpha", "ALPHA"), 0);
+    CM_ASSERT_EQUAL(cm_strcasecmp("alpha", "beta"), -1);
+    CM_ASSERT_EQUAL(cm_strcasecmp("alpha", "BETA"), -1);
+    CM_ASSERT_EQUAL(cm_strcasecmp("beta", "alpha"), +1);
+    CM_ASSERT_EQUAL(cm_strcasecmp("beta", "ALPHA"), +1);
+
+    refp = refstr; CM_ASSERT_EQUAL(cm_startswith(&refp, "alpha"), 1);
+    CM_ASSERT_EQUAL(strcmp(refp, "bet"), 0);
+    refp = refstr; CM_ASSERT_EQUAL(cm_startswith(&refp, "alpa"), 0);
+
+    CM_TEST_OK();
+}
+
+#endif  /* TESTING */
+
+
 void *sec_realloc(void *ptr, size_t size)
     /* Slightly more secure version of realloc() */
 {   size_t cnt, *memarr;
@@ -164,17 +208,17 @@ int get_randkey(unsigned char *buff, size_t len)
         { "/dev/urandom",   MEM_MAJOR, 9,       NULL },
         { NULL, 0, 0, NULL } };
     cm_sha1_ctxt_t *mdcontext=NULL;
-    unsigned char *mdval,*devbuff=NULL;
+    unsigned char *mdval, *devbuff=NULL;
     size_t pos, sz, blklen, mdlen;
-    int i,cnt,eflag=ERR_NOERROR;
+    int i, cnt, eflag=ERR_NOERROR;
     struct stat sbuff;
     pid_t pid;
     struct tms tbuff;
     clock_t clk;
-    static unsigned seed=17;
+    static unsigned seed=171;
 
 
-    /* try to find good-quality random-number devices: */
+    /* Try to find good-quality random-number devices: */
     for (i=0,cnt=0; devs[i].name!=NULL; ++i) {
         if (stat(devs[i].name, &sbuff) != 0) continue;
         if ((unsigned)major(sbuff.st_rdev) != devs[i].devmaj
@@ -192,24 +236,24 @@ int get_randkey(unsigned char *buff, size_t len)
     pid = getpid();
 
     /*  Combine multiple sources of entropy
-        (should still be effective if subset aren't viable): */
+        (should still be effective if a strict subset aren't viable): */
     pos = 0;
     while (pos < len) {
         mdcontext = cm_sha1_init();
 
-        /* fold-in entropy from random-number devices: */
+        /* Fold-in entropy from random-number devices: */
         for (i=0; devs[i].name!=NULL; ++i) {
             if (devs[i].fp == NULL) continue;
             (void)fread((void*)devbuff, (size_t)1, (size_t)blklen, devs[i].fp);
             cm_sha1_block(mdcontext, devbuff, blklen);
         }
 
-        /* fold-in existing bytes from key: */
+        /* Fold-in existing bytes from key: */
         if (pos > 0) {
             cm_sha1_block(mdcontext, buff, pos);
         }
 
-        /* fold-in some (weak) sources of entropy: */
+        /* Fold-in some (weak) sources of entropy: */
         cm_sha1_block(mdcontext, (unsigned char*)&pid, sizeof(pid));
         clk = times(&tbuff);
         cm_sha1_block(mdcontext, (unsigned char*)&clk, sizeof(clk));
@@ -219,10 +263,10 @@ int get_randkey(unsigned char *buff, size_t len)
         cm_sha1_final(mdcontext, &mdval, &mdlen);
 
         sz = ((pos + mdlen) > len ? (len - pos) : mdlen);
-        memcpy((void*)(buff + pos),(const void*)mdval, (size_t)sz);
+        memcpy((void*)(buff + pos),(const void*)mdval, sz);
 
         pos += sz;
-        seed = seed * 20 + 1;
+        seed = seed * 106 + 1283;
 
         cm_sha1_free(mdcontext);
         sec_free(mdval);
@@ -293,10 +337,12 @@ int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
     ssize_t plen=0;
     int eflag=ERR_NOERROR;
 
+    if (pw_ctxt != NULL && pw_ctxt->verify) verify |= 1;
+
     if (pw_ctxt == NULL || pw_ctxt->fd_pw_source == NULL) {
 #ifndef TESTING
         /* Read (+confirm) password from terminal: */
-	char prompt[BUFFSZ];
+        char prompt[BUFFSZ];
         snprintf(prompt, sizeof(prompt),
                 (isnew ? _("Enter new password for target \"%s\": ")
                     : _("Enter password for target \"%s\": ")),
@@ -431,9 +477,9 @@ int km_aug_verify(const unsigned char *buff, unsigned keylen,
 
 
 /*
- *  sha1 message-digest algorithm
+ *  SHA1 message-digest algorithm
  *  - based on "Cryptography - theory & practice" (2nd Ed.), DR Stinson, 2002
- *  (rather inefficient, but good enough for short messages)
+ *  (rather inefficient implementation, but good enough for short messages)
  */
 
 static const uint32_t
@@ -585,7 +631,7 @@ int ut_test_sha1()
     { "Cryptography - theory and practice, by Douglas R Stinson, Chapman & Hall/CRC",  "2e0e07901c8460bc57b0097a66c7086ed5e97808" },
     { NULL, NULL } };
 
-    CM_TEST_START("internal SHA1");
+    CM_TEST_START("Internal SHA1");
     CM_ASSERT_EQUAL(CM_SHA1_SIZE, 5 * sizeof(uint32_t));
 
     idx = 0;
@@ -610,8 +656,195 @@ int ut_test_sha1()
 
 #endif  /* TESTING */
 
+
+
+void cm_pwd_fortify(const char *passwd, unsigned iterations,
+                const unsigned char *salt, size_t saltlen,
+                unsigned char **key, size_t keylen)
+    /* Iteratively apply hashing algorithm to stretch & fortify password */
+{   cm_sha1_ctxt_t *mdcontext=NULL;
+    unsigned char q, *permsalt=NULL, *mdval=NULL, *mdval_prev=NULL;
+    size_t idx, pos, mdlen, pwlen, sz=0, newidx;
+    uint32_t cnt, r_val;
+    const uint32_t r_mult=421, r_inc=54773, r_mod=259200, r_scale=69317;
+
+    /* Initialize random-number seed from password vector: */
+    pwlen = strlen(passwd);
+    r_val = 1;
+    for (idx=0; idx<pwlen; ++idx) {
+        r_val = (r_val * r_scale + (uint32_t)passwd[idx]) % r_mod;
+    }
+
+    /* Assemble salt-vector which can be randomly permuted: */
+    if (salt != NULL && saltlen > 0) {
+        permsalt = sec_realloc((void*)permsalt, saltlen);
+        memcpy((void*)permsalt, (const void*)salt, saltlen);
+    } else {
+        if (saltlen == 0) saltlen = 16;
+        permsalt = sec_realloc((void*)permsalt, saltlen);
+        for (idx=0; idx<saltlen; ++idx) permsalt[idx] = idx;
+    }
+
+    *key = (unsigned char*)sec_realloc((void*)*key, keylen);
+    pos = 0;
+    while (pos < keylen) {
+        /* Iteratively apply hashing function to fortify password: */
+        for (cnt=0; cnt<iterations; ++cnt) {
+            mdcontext = cm_sha1_init();
+
+            if (cnt == 0) {
+                /* Permute salt vector: */
+                for (idx=0; idx<(saltlen-1); ++idx) {
+                    newidx = idx + (r_val % (saltlen - idx));
+
+                    q = permsalt[newidx];
+                    permsalt[newidx] = permsalt[idx];
+                    permsalt[idx] = q;
+
+                    r_val = (r_mult * r_val + r_inc) % r_mod;
+                }
+
+                /* Mix-in salt vector: */
+                cm_sha1_block(mdcontext, permsalt, saltlen);
+
+                /* Mix-in head of output vector: */
+                if (pos > 0) {
+                    cm_sha1_block(mdcontext, *key, pos);
+                }
+            } else {
+                /* Mix-in result of previous iteration: */
+                cm_sha1_block(mdcontext, mdval_prev, mdlen);
+            }
+
+            /* Mix-in password: */
+            cm_sha1_block(mdcontext, (const unsigned char*)passwd, pwlen);
+
+            cm_sha1_final(mdcontext, &mdval, &mdlen);
+
+            /* Merge (subset of) hash-code bytes into output key: */
+            if (cnt == 0) {
+                sz = ((pos + mdlen) > keylen ? (keylen - pos) : mdlen);
+                memcpy((void*)(*key + pos), (const void*)mdval, sz);
+            } else {
+                /* Mix with results from previous iterations: */
+                for (idx=0; idx<sz; ++idx) *(*key + pos + idx) ^= mdval[idx];
+            }
+
+            cm_sha1_free(mdcontext);
+
+            if (cnt > 0) sec_free(mdval_prev);
+            mdval_prev = mdval;
+            mdval = NULL;
+        }
+        pos += sz;
+        sec_free(mdval_prev);
+        mdval_prev = NULL;
+    }
+
+    sec_free(permsalt);
+}
+
+
+#ifdef TESTING
+
+int ut_pwfort()
+    /* Check that password-fortification behaves sensibly */
+{   unsigned idx, pos, r=0;
+    struct fwdbck {
+        char *passwd;
+        unsigned char *fwd, *bck;
+    } *fbs;
+    struct tcase {
+        const char *passwd;
+        const char *salt;
+        unsigned char front[4], back[4];
+    } tcases[] = {
+        { "Mary", "alpha",      { 0x46, 0x94, 0x0f, 0xb1 },
+                                { 0x8e, 0x0c, 0x6a, 0x6b } },
+        { "had", "beta",        { 0x99, 0xd7, 0x51, 0x0f },
+                                { 0x72, 0x23, 0x89, 0x2c } },
+        { "a", "gamma",         { 0x6a, 0xcf, 0x38, 0xa5 },
+                                { 0x57, 0x95, 0x6f, 0xcc } },
+        { "little", "delta",    { 0xe6, 0x53, 0x17, 0x26 },
+                                { 0x4c, 0x87, 0x05, 0xd7 } },
+        { "lamb", "epsilon",    { 0xc2, 0x16, 0xf6, 0x11 },
+                                { 0xf6, 0xaa, 0x02, 0x5d } },
+        { NULL, NULL,           {},
+                                {} } };
+    unsigned char *key=NULL;
+    const unsigned n_fbs = 64, n_its = 32;
+    const size_t pwlen = 13, saltlen = 17, keylen = 1024;
+
+    CM_TEST_START("Password fortification");
+
+    /* Generate set of keys from pseudo-random passwords: */
+    fbs = (struct fwdbck*)malloc(n_fbs * sizeof(struct fwdbck));
+    for (idx=0; idx<n_fbs; ++idx) {
+        fbs[idx].passwd = (char*)malloc(pwlen + 1);
+        fbs[idx].fwd = fbs[idx].bck = NULL;
+
+        for (pos=0; pos<pwlen; ++pos) {
+            fbs[idx].passwd[pos] = (char)('A' + (r++ % 61));
+        }
+        fbs[idx].passwd[pwlen] = '\0';
+
+        cm_pwd_fortify(fbs[idx].passwd, n_its, NULL, saltlen,
+                    &fbs[idx].fwd, keylen);
+
+        if (idx > 0) {
+            CM_ASSERT_DIFFERENT(0, memcmp((const void*)fbs[idx].fwd,
+                                    (const void*)fbs[idx-1].fwd, keylen));
+        }
+    }
+
+    /* Check that keys generated in reverse order match: */
+    for (idx=n_fbs; idx-->0; ) {
+        cm_pwd_fortify(fbs[idx].passwd, n_its, NULL, saltlen,
+                    &fbs[idx].bck, keylen);
+
+        CM_ASSERT_EQUAL(0, memcmp((const void*)fbs[idx].fwd,
+                                (const void*)fbs[idx].bck, keylen));
+
+        /* Check that key doesn't contain trivial repetitions: */
+        pos = 0;
+        while ((pos + 2 *CM_SHA1_SIZE) < keylen) {
+            CM_ASSERT_DIFFERENT(0, memcmp((const void*)(fbs[idx].fwd + pos),
+                                        (const void*)(fbs[idx].fwd + pos + CM_SHA1_SIZE), (size_t)CM_SHA1_SIZE));
+            pos += CM_SHA1_SIZE;
+        }
+
+        sec_free((void*)fbs[idx].fwd);
+        sec_free((void*)fbs[idx].bck);
+        free((void*)fbs[idx].passwd);
+    }
+    free((void*)fbs);
+
+    /* Check known test cases: */
+    for (idx=0; tcases[idx].passwd!=NULL; ++idx) {
+        cm_pwd_fortify(tcases[idx].passwd, n_its,
+                    (unsigned char*)tcases[idx].salt, strlen(tcases[idx].salt),
+                    &key, keylen);
+#if 0
+        fprintf(stderr, "%s: ", tcases[idx].passwd);
+        for (pos=0; pos<keylen; ++pos) fprintf(stderr, "0x%02x, ", key[pos]);
+        fprintf(stderr, "\n\n");
+#endif
+
+        CM_ASSERT_EQUAL(0, memcmp((const void*)key,
+                                (const void*)tcases[idx].front, (size_t)4));
+        CM_ASSERT_EQUAL(0, memcmp((const void*)(key + keylen - 4),
+                                (const void*)tcases[idx].back, (size_t)4));
+    }
+
+    if (key != NULL) sec_free((void*)key);
+
+    CM_TEST_OK();
+}
+
+#endif  /* TESTING */
+
 /* vim: set ts=4 sw=4 et: */
 
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
