@@ -1,7 +1,7 @@
 /*
  *  declarations for miscellaneous utilities for cryptmount
  *  $Revision: 134 $, $Date: 2007-02-19 17:18:07 +0000 (Mon, 19 Feb 2007) $
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
 
 /*
@@ -30,10 +30,11 @@
 #include <stdio.h>
 
 typedef struct km_pw_context {
-    FILE *fd_pw_source;         /* stream from which to read passwords */
-    unsigned debug_level;	/* verbosity of debugging information */
+    FILE *fd_pw_source;         /* Stream from which to read passwords */
+    int verify;                 /* Always verify passwords from terminal */
+    unsigned debug_level;       /* Verbosity of debugging information */
 #ifdef TESTING
-    const char *argpasswd[2];   /* password(s) passed via command-line */
+    const char *argpasswd[2];   /* Password(s) passed via command-line */
 #endif
 } km_pw_context_t;
 
@@ -41,6 +42,7 @@ typedef struct km_pw_context {
 int cm_path(char **buff, const char *file);
 char *cm_strdup(const char *orig);
 int cm_strcasecmp(const char *s1, const char *s2);
+int cm_startswith(const char **str, const char *prefix);
 
 void *sec_realloc(void *ptr, size_t size);
 void mem_cleanse(unsigned char *addr, size_t sz);
@@ -69,10 +71,44 @@ void cm_sha1_block(cm_sha1_ctxt_t *ctxt, const unsigned char *buff, size_t len);
 void cm_sha1_final(cm_sha1_ctxt_t *ctxt, unsigned char **mdval, size_t *mdlen);
 void cm_sha1_free(cm_sha1_ctxt_t *ctxt);
 
+void cm_pwd_fortify(const char *passwd, unsigned iterations,
+                const unsigned char *salt, size_t saltlen,
+                unsigned char **key, size_t keylen);
+
+
+static inline uint16_t pack_uint16(const unsigned char *buff) {
+    return (((uint16_t)buff[1]) << 8) | ((uint16_t)buff[0]); }
+
+static inline void unpack_uint16(unsigned char *buff, const uint16_t val) {
+    buff[0] = (val & 0x00ff); buff[1] = (val & 0xff00) >> 8; }
+
+static inline uint32_t pack_uint32(const unsigned char *buff) {
+    return (((uint32_t)buff[3]) << 24) | (((uint32_t)buff[2]) << 16) \
+            | (((uint32_t)buff[1]) << 8) | ((uint32_t)buff[0]); }
+
+static inline void unpack_uint32(unsigned char *buff, const uint32_t val) {
+    buff[0] = (val & 0x000000ff); buff[1] = (val & 0x0000ff00) >> 8;
+    buff[2] = (val & 0x00ff0000) >> 16; buff[3] = (val & 0xff000000) >> 24; }
+
+
+static inline int cm_fread(void *buff, size_t nbytes, FILE *stream) {
+    /* Read bytes from file, returning 0 on success */
+    return (fread(buff, nbytes, (size_t)1, stream) != 1);
+}
+
+
+static inline int cm_fwrite(const void *buff, size_t nbytes, FILE *stream) {
+    /* Write buffer to file, returning 0 on success */
+    return (fwrite(buff, nbytes, (size_t)1, stream) != 1);
+}
+
+
+
+
 #endif  /* _UTILS_H */
 
 /* vim: set ts=4 sw=4 et: */
 
 /*
- *  (C)Copyright 2005-2008, RW Penney
+ *  (C)Copyright 2005-2009, RW Penney
  */
